package ejava.corba.dynamic;



import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.RequestProcessingPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.DynamicImplementation;


public class SampleDynamic extends DynamicImplementation
{

  private String            returnValue;
  private String[]          ids;
  private POA               poa;
  private org.omg.CORBA.ORB orb;
  private String            objectName;
  private final String      OPERATION_NAME    = "sayHello";
  private final String      IN_PARAMETER_NAME = "hello";

  public SampleDynamic(String name, POA poaRef, org.omg.CORBA.ORB orb)
  {

    this.objectName = name;
    this.poa        = poaRef;
    this.orb        = orb;
    ids             = new String[]{ "IDL:SampleDynamic:1.0" };
  }

  public void invoke(org.omg.CORBA.ServerRequest serverRequest)
  {

    // make sure the operation name is correct
    if( !serverRequest.operation().equals(OPERATION_NAME)) {
      throw new org.omg.CORBA.BAD_OPERATION();
    }

    // create an empty name value parameter list
    org.omg.CORBA.NVList parameters = orb.create_list(0);

    // create an any for ParameterName
    org.omg.CORBA.Any nameOfParameter = orb.create_any();

    // any's type is String
    nameOfParameter.type(
      orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string));

    // add "in" the parameter name to the parameter list
    parameters.add_value(IN_PARAMETER_NAME, nameOfParameter,
                         org.omg.CORBA.ARG_IN.value);

    // get the parameter and values from the request
    serverRequest.arguments(parameters);

    // get received value from parameter
    String receivedValue = nameOfParameter.extract_string();

    // invoke sayHello method
    String returnValue = this.sayHello(receivedValue);

    // create an any for the reply
    org.omg.CORBA.Any stringAny = orb.create_any();

    stringAny.insert_string(returnValue);

    // set the request's result
    serverRequest.set_result(stringAny);
  }

  String sayHello(String receivedValue)
  {
    return "You sent me :" + receivedValue;
  }

  public String[] _ids()
  {
    return ids;
  }

  public String _object_name()
  {
    return this.objectName;
  }

  public String[] _all_interfaces(POA poa, byte[] objectId)
  {

    //Used by the ORB to obtain complete type information from
    // the servant. For this example we can return null;
    return null;
  }
}
