package ejava.ejb.entity.customer;

import javax.ejb.EJBLocalHome;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import java.util.Collection;

/**
 * This is the home interface for the Customer EntityBean.
 *
 */
public interface CMRCustomerLocalHome extends EJBLocalHome
{

  /**
   * This method corresponds to the ejbCreate method in the bean
   * CustomerEJBean.java.
   * When this method is called, the container will call ejbCreate()
   * on the EntityBean.
   *
   *
   * @param customerID         Customer ID
   * @param firstName          First Name of the customer
   * @param lastName           Last Name of the Customer
   * @param middleName,        middle Name of the Customer
   * @param email,             email of the Customer
   * @param  address_1,        address_1 of the Customer
   * @param address_2,         address_2 of the Customer
   * @param  city,             city of the Customer
   * @param state,             state of the Customer
   * @param zipCode,           zipCode of the Customer
   * @param phone,             phone of the Customer
   * @param password           password of the Customer
   *
   */
  public CMRCustomerLocal create(Integer customerID, String firstName,
                                 String lastName, String middleName,
                                 String email, String address_1,
                                 String address_2, String city,
                                 String state, String zipCode,
                                 String phone) throws CreateException;

  /**
   * Given a Primary Key, refreshes the EJBean from
   * the persistent storage in our case it is RDBMS
   * @param customeID         Primary Key
   * @return                  Customer
   */
  public CMRCustomerLocal findByPrimaryKey(Integer customeID)
      throws FinderException;

  /**
   * Finds a Customer given their email address
   *
   * @param email         Email Address of the customer
   */
  public CMRCustomerLocal findByEmail(String email)
      throws  FinderException;

  public Collection findAll()
      throws FinderException;

  public Collection findCustomersWithOrdersInRange(double startValue,
                                                   double endVlaue)
      throws FinderException;

  public Collection findAllCustomersWhoBought(String shirtsSize)
      throws FinderException;

  public Collection findAllCustomersWhoDidNotBuy(String shirtSize)
      throws FinderException;

  public Collection findCustomersWhoseFirstNameLike()
      throws FinderException;

  public abstract Collection findCustomersWhoDoNotHaveOrders()
      throws FinderException;

  public abstract Collection findCustomersWhoAreFromCapitalArea()
      throws FinderException;

}

