package ejava.ejb.mdb;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.EJBException;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import ejava.ejb.session.ordermgr.OrderManagerLocal;
import ejava.ejb.session.ordermgr.OrderManagerLocalHome;
import ejava.servlets.Order;
import ejava.ejb.EJBHelper;

public class BatchOrderEJBean
	implements MessageDrivenBean, MessageListener {

	// Message-driven context
	private MessageDrivenContext messageDrivenContext;
	// Handle to the order manager home interface (local)
	private OrderManagerLocalHome orderManagerHome;

	/**
	 * Standard ejbCreate() method. Simply print out a message
	 * and lookup a handle to a local order manager home
	 * interface.
	 */
	public void ejbCreate() {
		System.out.println("MDB: ejbCreate called");
		try {
			lookupOrderManagerHome();
		} catch (NamingException ne) {
			ne.printStackTrace();
		}
	}

	/**
	 * Standard ejbRemove() method. Simply print out a message.
	 */
	public void ejbRemove() {
		System.out.println("MDB: ejbRemove called");
	}

	/**
	 * Standard setMessageDrivenContext() method. Simply print 
	 * out a message.
	 */
	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		System.out.println("MDB: setMessageDrivenContext called");
		this.messageDrivenContext = ctx;
	}

	/**
   * Receives an ObjectMessage, gets the order from the 
   * JMS message, and then delegates the call to the
   * OrderManager session bean for creating the order.
	 */
	public void onMessage(Message receivedMessage) {
		try {
			// If an object message is received...
			if (receivedMessage instanceof ObjectMessage) {
				ObjectMessage message = (ObjectMessage) receivedMessage;
				System.out.println("MDB: onMessage called");
				// Get the order from the message and create an order
				Order order = (Order) message.getObject();
				createAnOrder(order);
			} else {
				System.out.println(
					"This bean can consume only ObjectMessage types");
			}
		} catch (JMSException ex) {
			ex.printStackTrace();
		} catch (CreateException ce) {
			ce.printStackTrace();
			System.out.println("Failed to Create Order:");
		}
	}

	/**
   * This private method creates a new order object by
   * inducing the OrderManager session bean to create the order.
   */
	private Order createAnOrder(Order order) throws CreateException {
		System.out.println("creating order...");
		OrderManagerLocal orderManager = this.orderManagerHome.create();
		Integer newOrderID = orderManager.createNewOrder(order);
		order.setOrderID(newOrderID.intValue());
		System.out.println("Newly Created Order ID :" + newOrderID);
		return order;
	}

	/** Lookup OrderManagerHome object */
	private void lookupOrderManagerHome() throws NamingException {
		if (orderManagerHome == null) {
			InitialContext context = new InitialContext();
			orderManagerHome =
				(OrderManagerLocalHome) context.lookup(
					EJBHelper.ORDER_MANAGER_REFNAME);
			context.close();
		}
	}
}
