package ejava.ejb.security.basic;

/**
 * This example class serves as a code snippet file
 * only and is not meant to represent an actual
 * working example of EJB security.
 * It serves as an illustrative example for
 * making security calls from within a bean.
 */

import javax.ejb.*;
import javax.naming.*;
import java.util.*;
import ejava.ejb.session.ordermgr.*;
import ejava.ejb.entity.customer.*;

public class SecureOrderManagerEJBean extends OrderManagerEJBean {

  /**
   * Get customer username (i.e. email) based on customer ID
   */
  private String getUserName(int customerID){
    String userName = "";
    try{
      CustomerLocal customer 
        = getCustomerHome().findByPrimaryKey(new Integer(customerID));
      userName = customer.getEmail();
    }catch(Exception e){
      System.out.println("Username not found " + e.getMessage());
    }
    return userName;
  }

  /**
    * Query for the orders in the DBMS and return them.
    */
  public Vector queryOrders(int customerID) {
    Vector orders = null;
    try {
      // Get a user name associated with this customer ID
      String customerUserName = this.getUserName(customerID);
      
      // Get the EJB context for this bean
      EJBContext ejbContext = this.getSessionContext();
      
      // Get principal from context and principal name
      java.security.Principal caller = ejbContext.getCallerPrincipal();
      String callerUserName = caller.getName();

      // If not in manager role and not the user associated with
      //   this order, then deny access...
      if ((ejbContext.isCallerInRole("manager") == true)
        || (callerUserName.equalsIgnoreCase(customerUserName))) {
        // Allow access to do security-critical stuff here...
        //   ...which in this case is to call superclass method
        orders = super.queryOrders(customerID);
      } else {
        // Access denied. Audit this event and return...
        System.out.println("**** ILLEGAL ACCESS ATTEMPT ****"
                          + "\n  OrderManager.queryOrders(int)"
                          + "\n  Caller Name; " + callerUserName
                          + "\n  Customer Name " + customerUserName);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return orders;
  }  
}