package ejava.ejb.security.jaas;

import java.security.PrivilegedAction;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Date;
import javax.rmi.PortableRemoteObject;

import ejava.ejb.EJBHelper;
import ejava.servlets.TShirt;
import ejava.servlets.Order;
import ejava.servlets.Item;
import ejava.servlets.Customer;
import ejava.servlets.Address;
import ejava.ejb.session.ordermgr.OrderManagerHome;
import ejava.ejb.session.ordermgr.OrderManager;

public class TestAction implements PrivilegedAction {
	private static String userName;
	private static String password;
	private static int queryCustomerID = 101;
	private static int queryOrderID = 1001;

  /**
   * Implement standard call for PrivelegedAction
   */
	public Object run() {
		try {
      // Method performs some ap-specific functions
			processApSpecificFunctionality();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

  /**
   * Calls security-critical order management code
   */
	public void processApSpecificFunctionality() {
		try {
			Context context = new InitialContext();
			System.out.println(
				" looking for :"
					+ EJBHelper.ORDER_MANAGER_HOME
					+ " context:"
					+ context);
			Object objectRef = context.lookup(EJBHelper.ORDER_MANAGER_HOME);
			OrderManagerHome orderManagerHome =
				(OrderManagerHome) PortableRemoteObject.narrow(
					objectRef,
					ejava.ejb.session.ordermgr.OrderManagerHome.class);
			OrderManager orderManager = orderManagerHome.create();

			Vector orders = orderManager.queryOrders(queryCustomerID);
			System.out.println("Received Orders :" + orders.getClass());
			for (int i = 0; i < orders.size(); i++) {
				Order order = (Order) orders.elementAt(i);
				System.out.println("Received Order : " + i + ": Order Detail");
				printOrder(order);
			}
			// Query # of items
			int nItems = orderManager.numberOfItemsInAnOrder(queryOrderID);
			System.out.println(" Number of Items in an Order :" + nItems);

			// Avarage OrderValue of a Customer
			float avgValue =
				orderManager.averageOrderValueOfACustomer(queryCustomerID);
			System.out.println(
				" Average Order Value of a Customer :" + avgValue);
			Customer customer = orderManager.getCustomer(queryOrderID);
			System.out.println("Received Customer :" + customer);
			// Create a New Order
			Order order = new Order();
			Integer newOrderID = createNewOrder(orderManager);

			System.out.println(" New Order ID :" + newOrderID);
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	private void printOrder(Order order) {
		System.out.println("ID :" + order.getOrderID());
		System.out.println("orderDate :" + order.getOrderDate());
		System.out.println(
			"shipInstruct :" + order.getShippingInstructions());
		System.out.println("shipDate :" + order.getShipDate());
		System.out.println("shipWeight :" + order.getShipWeight());
		System.out.println("paidDate :" + order.getPaidDate());
	}

	private Integer createNewOrder(OrderManager orderManager)
		throws RemoteException {
		Order order = new Order();
		// Order is created today and paid today and shipped today.
		order.setOrderDate(new Date());
		order.setPaidDate(new Date());
		order.setShippingInstructions("Leave at Door");
		order.setShipWeight(10.8);
		order.setShipCharge(32.54);
		order.setShipDate(new Date());
		// The customer info the order is beeing created for
		Customer customer = new Customer();
		customer.setCustomerID("" + queryCustomerID);
		order.setCustomer(customer);
		order.setBillingAddress(createAnAddress());
		order.setShippingAddress(createAnAddress());
		order.setItems(createItems());
		return orderManager.createNewOrder(order);
	}

	private Address createAnAddress() {
		Address address = new Address();
		address.setAddress_1(" 2.1 EJB Way");
		address.setAddress_2(" 1");
		address.setCity(" J2EE ");
		address.setState("CA");
		address.setZipCode("14000");
		return address;
	}

	private TShirt[] createItems() {
		TShirt itemOne = new TShirt();
		itemOne.setQuantity(2);
		itemOne.setTotalPrice(21.0);
		itemOne.setSize("XL");
		itemOne.setColor("White");
		itemOne.setUnitPrice(10.5);

		TShirt itemTwo = new TShirt();
		itemTwo.setQuantity(2);
		itemTwo.setTotalPrice(21.0);
		itemTwo.setSize("L");
		itemTwo.setColor("White");
		itemTwo.setUnitPrice(10.5);
		TShirt[] createdItems = { itemOne, itemTwo };
		return createdItems;
	}
}
