package ejava.ejb.session.customermgr;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.util.Vector;
import ejava.servlets.Customer;


/**
 *  The public interface methods to the CustomerManagerEJBean SessionBean,
 *  which is used to query for info about Customers
 */
public interface CustomerManager extends EJBObject
{
   /**
    * Query All Customers
    */
   public Vector getAllCustomers() throws RemoteException;

   /**
    * Get Customer detail Information
    */
   public Customer getCustomerDetail(int customerID)
      throws RemoteException;

   /**
    * Get Customers whose orders are in the specified range
    */
   public Vector getCustomersWithOrdersInRange(double minValue,
                                               double maxValue)
      throws RemoteException;

   /**
    * Get Customers who bought a certain size shirts.
    */
   public Vector getCustomersWhoBought(String shirtsSize)
     throws RemoteException;

   /**
    * Get Customers who did not buy a certain size shirts.
    */
   public Vector getCustomersWhoDidNotBuy(String shirtsSize)
     throws RemoteException;

   /**
    * Get Customers who did not buy a certain size shirts.
    */
   public Vector getCustomersWhoseFirstNameLike()
     throws RemoteException;

   /**
    * Get Customers who did not have any Orders
    */
   public Vector getCustomersWhoDoNotHaveOrders()
     throws RemoteException;

   /**
    * Get Customers who are from the USA metro D.C. region
    */
   public Vector getCustomersWhoAreFromCapitalArea()
     throws RemoteException;

    /**
    * Create New Customer
    */
   public Integer createNewCustomer(Customer customer)
     throws RemoteException;

   /**
    * performs authentication on user
    */
   public Customer login(String email, String password)
       throws RemoteException, FailedToLoginException;

   /**
    * registers a new Customer
    */
   public Integer register(Customer customer)
       throws RemoteException, FailedToRegisterException;

   /**
    * sends the user a password reminder email
    */
   public void remindPassword(String email) throws RemoteException;

}


