package ejava.ejb.session.customermgr;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import java.rmi.RemoteException;
import javax.ejb.FinderException;
import javax.rmi.PortableRemoteObject;
import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;
import ejava.ejb.EJBHelper;

import ejava.ejb.entity.tshirt.CMRTShirtLocal;
import ejava.ejb.entity.order.CMROrderLocal;
import ejava.ejb.entity.order.CMROrderLocalHome;
import ejava.ejb.entity.item.CMRItemLocal;
import ejava.ejb.entity.customer.CMRCustomerLocalHome;
import ejava.ejb.entity.customer.CMRCustomerLocal;
import ejava.ejb.entity.authentication.AuthenticationLocal;
import ejava.ejb.entity.authentication.AuthenticationLocalHome;
import ejava.servlets.TShirt;
import ejava.servlets.Item;
import ejava.servlets.Order;
import ejava.servlets.Customer;
import ejava.servlets.Address;

public class CustomerManagerEJBean implements SessionBean {
  private SessionContext sessionContext;
  protected CMRCustomerLocalHome customerHome;
  protected AuthenticationLocalHome authenticationHome;

  /**
   *  Empty default implementation of SessionBean interface
   */
  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  /**
   * This method is required by the EJB Specification for
   * implementing a SessionBean to set the context.
   */
  public void setSessionContext(SessionContext sessionContext)
    throws RemoteException {
    this.sessionContext = sessionContext;
    try {
      lookupCustomerHome();
      lookupAuthenticationHome();
    } catch (NamingException ne) {
      throw new RemoteException(ne.getMessage());
    }
  }

  /**
   * This method corresponds to the create method in the home interface
   * OrderManagerHome.
   */
  public void ejbCreate() throws CreateException {
  }

  public void lookupCustomerHome() throws NamingException {
    if (this.customerHome == null) {
      InitialContext context = new InitialContext();
      customerHome =
        (CMRCustomerLocalHome) context.lookup(
          EJBHelper.CUSTOMER_REFNAME);
      context.close();
    }
  }

  public Vector getAllCustomers() throws EJBException {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing findAll");
      Collection customers = this.customerHome.findAll();
      Iterator it = customers.iterator();
      while (it.hasNext()) {

        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }

    return returnCustomers;
  }

  private Customer fillReturnCustomer(CMRCustomerLocal customerLocal) {
    Address address =
      new Address(
        customerLocal.getAddress_1(),
        customerLocal.getAddress_2(),
        customerLocal.getCity(),
        customerLocal.getState(),
        customerLocal.getZipCode());
    Customer customer =
      new Customer(
        customerLocal.getFirstName(),
        customerLocal.getLastName(),
        customerLocal.getMiddleName(),
        address,
        customerLocal.getPhone(),
        customerLocal.getEmail(),
        null);
    customer.setCustomerID(customerLocal.getCustomerID().toString());
    return customer;

  }

  public Customer getCustomerDetail(int customerID) {
    Customer returnValue = null;
    try {
      CMRCustomerLocal customerLocal =
        this.customerHome.findByPrimaryKey(new Integer(customerID));
      returnValue = fillReturnCustomer(customerLocal);
    } catch (FinderException fe) {
      throw new EJBException(fe.getMessage());
    }
    return returnValue;
  }

  /**
   * Get Customers whose orders or in range
   */
  public Vector getCustomersWithOrdersInRange(
    double minValue,
    double maxValue) {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing findCustomersWithOrdersInRange");
      Collection customers =
        this.customerHome.findCustomersWithOrdersInRange(
          minValue,
          maxValue);
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }
  /**
   * Get Customers who bought a certain size shirts.
   */
  public Vector getCustomersWhoBought(String shirtsSize) {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing findAllCustomersWhoBought");
      Collection customers =
        this.customerHome.findAllCustomersWhoBought(shirtsSize);
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }
  /**
   * Get Customers who did not buy a certain size shirts.
   */
  public Vector getCustomersWhoDidNotBuy(String shirtsSize) {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing findAllCustomersWhoDidNotBuy");
      Collection customers =
        this.customerHome.findAllCustomersWhoDidNotBuy(shirtsSize);
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }
  /**
   * Get Customers who did not buy a certain size shirts.
   */
  public Vector getCustomersWhoseFirstNameLike() {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing findCustomersWhoseFirstNameLike");
      Collection customers =
        this.customerHome.findCustomersWhoseFirstNameLike();
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }

  /**
   * Get Customers who did not have any Orders
   */
  public Vector getCustomersWhoDoNotHaveOrders() {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing getCustomersWhoDoNotHaveOrders");
      Collection customers =
        this.customerHome.findCustomersWhoDoNotHaveOrders();
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }

  /**
  * Get Customers who are from any of these states
  */
  public Vector getCustomersWhoAreFromCapitalArea() {
    Vector returnCustomers = null;
    try {
      System.out.println("Testing getCustomersWhoAreFromCapitalArea");
      Collection customers =
        this.customerHome.findCustomersWhoAreFromCapitalArea();
      Iterator it = customers.iterator();
      while (it.hasNext()) {
        CMRCustomerLocal customer = (CMRCustomerLocal) it.next();
        System.out.println(
          "Returning Customer ID:" + customer.getCustomerID());
        if (returnCustomers == null)
          returnCustomers = new Vector();
        returnCustomers.addElement(fillReturnCustomer(customer));
      }
    } catch (FinderException finderException) {
      throw new EJBException(finderException.getMessage());
    }
    return returnCustomers;
  }

  /**
   * Create a new Customer
   */
  public Integer createNewCustomer(Customer customer) {
    Integer returnCustomerID = null;
    try {
      returnCustomerID = EJBHelper.getNextCustomerID();
      Address address = customer.getAddress();
      CMRCustomerLocal customerLocal =
        this.customerHome.create(
          returnCustomerID,
          customer.getFirstName(),
          customer.getLastName(),
          customer.getMiddleName(),
          customer.getEmail(),
          address.getAddress_1(),
          address.getAddress_2(),
          address.getCity(),
          address.getState(),
          address.getZipCode(),
          customer.getPhone());
      System.out.println("Created Customer :" + returnCustomerID);
      Integer nextAuthenticationID =
        EJBHelper.getNextAuthenticationID();
      System.out.println("Created Customer :" + returnCustomerID);
      AuthenticationLocal authentication =
        this.authenticationHome.create(
          nextAuthenticationID,
          returnCustomerID,
          "PASSWORD",
          customer.getPassword(),
          "ACTIVE");
      System.out.println(
        "Setting Authentication :" + nextAuthenticationID);
      customerLocal.setAuthentication(authentication);
      System.out.println(
        "Finished Setting Authentication:" + nextAuthenticationID);
    } catch (CreateException ce) {
      throw new EJBException(ce.getMessage());
    } catch (SQLException se) {
      throw new EJBException(
        "Failed to get Next Customer #" + se.getMessage());
    } catch (NamingException ne) {
      throw new EJBException(
        "Failed to get Next Customer #" + ne.getMessage());
    }
    return returnCustomerID;
  }

  private void lookupAuthenticationHome() throws NamingException {
    // Create initial context
    InitialContext initialContext = new InitialContext();
    // Lookup AuthenticationHomeHome
    authenticationHome =
      (AuthenticationLocalHome) initialContext.lookup(
        EJBHelper.AUTHENTICATION_REFNAME);
    // Close context
    initialContext.close();
  }

  /**
   * authenticates the user with email and password
   */
  public Customer login(String email, String password)
    throws FailedToLoginException {
    try {
      CMRCustomerLocal customerBean = customerHome.findByEmail(email);
      AuthenticationLocal authentication =
        customerBean.getAuthentication();
      if (!authentication.getPassword().equals(password)
        && !authentication.getActive().equals("ACTIVE")) {
        String message = "Failed: no user with that eamil and password";
        throw new FailedToLoginException(message);
      }
      // set values for the customer object for the Client
      Customer clientCustomer = fillReturnCustomer(customerBean);
      System.out.println("Got Customer" + clientCustomer);
      return clientCustomer;
    } catch (FinderException finderException) {
      throw new FailedToLoginException(finderException.getMessage());
    }
  }
  /**
    * creates new Customer Entity bean with the user entered values
    */
  public Integer register(Customer customer)
    throws FailedToRegisterException {
    System.out.println("Came To Register");
    Integer newCustomerID = null;
    try {
      newCustomerID = createNewCustomer(customer);
    } catch (EJBException ejbException) {
      throw new FailedToRegisterException(ejbException.getMessage());
    }
    return newCustomerID;
  }

  /**
   * gets the user's password and send it to the user in an email
   */
  public void remindPassword(String email) {
    try {
      System.out.println("Sending email to: " + email);

      // Lookup local interface and password
      CMRCustomerLocal customerBean = customerHome.findByEmail(email);
      String password = customerBean.getAuthentication().getPassword();

      // Send mail using MailSender utility given email and password
      MailSender.sendMail(email, password);
    } catch (FinderException finderException) {
      finderException.printStackTrace();
    }
  }

}
