package ejava.ejb.session.customermgr;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import ejava.ejb.EJBHelper;
import ejava.servlets.Customer;
import ejava.servlets.Address;


public class TestClient
{
  private static String userName;
  private static String password;
  private static int queryCustomerID = 101;
  private static double queryOrderMinValue = 10.0;
  private static double queryOrderMaxValue = 500.0;
  private static String querySize = "XL";


  public static void printCustomer(Customer customer)
  {
    System.out.println(customer);
  }

  public static void main(String[] args)
  {
    try{
      Context context = new InitialContext();
      System.out.println(" looking for :" +
            EJBHelper.CUSTOMER_MANAGER_HOME+" context:"+context);
      Object objectRef =
        context.lookup(EJBHelper.CUSTOMER_MANAGER_HOME);
      CustomerManagerHome customerManagerHome =
        (CustomerManagerHome)PortableRemoteObject.narrow(objectRef,
          ejava.ejb.session.customermgr.CustomerManagerHome.class);
      CustomerManager customerManager = customerManagerHome.create();
      // Get All Customers
      Vector customers = customerManager.getAllCustomers();
      System.out.println("<<CustomerManager.getAllCustomers()>>");
      for(int i = 0; i<customers.size(); i++){
        Customer customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }
      System.out.println(
         "<<CustomerManager.getCustomerDetail("+ queryCustomerID+")>>");
      //Get Customer by Customer ID
      Customer customer =
         customerManager.getCustomerDetail(queryCustomerID);
      printCustomer(customer);

      // Get All Customers whose orders value is in the Given Range
      customers =
         customerManager.getCustomersWithOrdersInRange(
                  queryOrderMinValue,queryOrderMaxValue);
      System.out.println(
         "<<CustomerManager.getCustomersWithOrdersInRange()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }

      // Get All Customers who bought a given size Shirts
      customers =
         customerManager.getCustomersWhoBought(querySize);
      System.out.println(
         "<<CustomerManager.getCustomersWhoBought()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }

      // Get All Customers who did not buy a given size Shirts
      customers =
         customerManager.getCustomersWhoDidNotBuy(querySize);
      System.out.println(
         "<<CustomerManager.getCustomersWhoDidNotBuy()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }

      // Get All Customers whose first name is like given pattern
      customers =
         customerManager.getCustomersWhoseFirstNameLike();
      System.out.println(
         "<<CustomerManager.getCustomersWhoseFirstNameLike()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println(
         "Received Customer : " +i +": Customer Detail");
        printCustomer(customer);
      }

      // Get All Customers who does not have any orders
      customers =
         customerManager.getCustomersWhoDoNotHaveOrders();
      System.out.println(
         "<<CustomerManager.getCustomersWhoDoNotHaveOrders()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }

      // Get All Customers who are from any of the three given states
      customers =
         customerManager.getCustomersWhoAreFromCapitalArea( );
      System.out.println(
         "<<CustomerManager.getCustomersWhoAreFromCapitalArea()>>");
      for(int i = 0; i<customers.size(); i++){
        customer = (Customer)customers.elementAt(i);
        System.out.println("Received Customer : " +i +
                              ": Customer Detail");
        printCustomer(customer);
      }

      // Create new Customer
      Integer newCustomerID =
         createNewCustomer(customerManager);
      System.out.println(
         "<<CustomerManager.createNewCustomer()>>");
      System.out.println("New Customer ID : " +newCustomerID);
    }
    catch(Exception exception){
      exception.printStackTrace();
    }
  }

  public static Integer createNewCustomer(
   CustomerManager customerManager)
   throws RemoteException
  {
    Address address = new Address();
    address.setAddress_1(" 1 EJB Way");
    address.setAddress_2(" Suite 1");
    address.setCity(" J2EE ");
    address.setState("VA");
    address.setZipCode("14000");
    Customer customer = new Customer();
    customer.setFirstName("Bill");
    customer.setLastName("Nelson");
    customer.setEmail("bn@bnelson.com");
    customer.setPhone("7871230000");
    customer.setPassword("test");
    customer.setAddress(address);
    Integer returnValue = customerManager.createNewCustomer(customer);
    return returnValue;
  }
}

