package ejava.ejb.session.shoppingcart;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import java.rmi.RemoteException;
import javax.ejb.FinderException;
import javax.rmi.PortableRemoteObject;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;
import java.util.Date;
import ejava.ejb.EJBHelper;

import ejava.ejb.entity.tshirt.CMRTShirtLocalHome;
import ejava.ejb.entity.tshirt.CMRTShirtLocal;
import ejava.ejb.session.ordermgr.OrderManagerLocalHome;
import ejava.ejb.session.ordermgr.OrderManagerLocal;
import ejava.ejb.entity.customer.CMRCustomerLocalHome;
import ejava.ejb.entity.customer.CMRCustomerLocal;

import ejava.servlets.Address;
import ejava.servlets.TShirt;
import ejava.servlets.Item;
import ejava.servlets.Customer;
import ejava.servlets.Order;

public class ShoppingCartEJBean implements SessionBean {
  // Session context
  private SessionContext sessionContext;
  // Local entity bean home interfaces
  private CMRTShirtLocalHome tshirtHome;
  private OrderManagerLocalHome orderManagerHome;
  private CMRCustomerLocalHome customerHome;
  // Shopping cart items (stateful client specific state)
  private Hashtable itemsInTheCart;
  // Time duration constants
  private static final long FIVE_DAYS = 432000000;
  private static final long THIRTY_DAYS = FIVE_DAYS * 6;

  // Standard interfaces
  public void setSessionContext(SessionContext aCtx) {
    sessionContext = aCtx;
    try {
      lookupCustomerHome();
      lookupTShirtHome();
      lookupOrderManagerHome();
    } catch (NamingException ne) {
      throw new EJBException(ne.getMessage());
    }
  }
  /**
   *  Default empty implementations of SessionBean interface
   */
  public void ejbCreate()
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbCreate(Hashtable initialCart) // Initial cart data
  {
    itemsInTheCart = initialCart;
  }

  /** Lookup Customer Home object */
  private void lookupCustomerHome() throws NamingException {
    if (customerHome == null) {
      InitialContext context = new InitialContext();
      customerHome =
        (CMRCustomerLocalHome) context.lookup(
          EJBHelper.CUSTOMER_REFNAME);
      context.close();
    }
  }

  /** Lookup TShirt Home object */
  private void lookupTShirtHome() throws NamingException {
    if (tshirtHome == null) {
      InitialContext context = new InitialContext();
      tshirtHome =
        (CMRTShirtLocalHome) context.lookup(EJBHelper.TSHIRT_REFNAME);
      context.close();
    }
  }

  /** Lookup Order Home object */
  private void lookupOrderManagerHome() throws NamingException {
    if (orderManagerHome == null) {
      InitialContext context = new InitialContext();
      orderManagerHome =
        (OrderManagerLocalHome) context.lookup(
          EJBHelper.ORDER_MANAGER_REFNAME);
      context.close();
    }
  }

  /**
   * Add a particular item identified by ID and quantity to session
   */
  public void addAnItemToTheCart(int shirtID, int quantity) {
    // If no Hashtable created make one
    if (itemsInTheCart == null) {
      itemsInTheCart = new Hashtable();
    }
    try {
      // Put the item ID and quantity into Hashtable (session state)
      itemsInTheCart.put(new Integer(shirtID), new Integer(quantity));
    } catch (Exception exception) {
      exception.printStackTrace();
    }
  }

  /**
   * For each item in session, return Vector of associated TShirts
   */
  public Vector getSelectedItemsInTheCart() {
    // Create return Vector
    Vector returnItemsInTheCart = new Vector();

    // For each ID in Hashtable (session state)
    Enumeration shirtIDs = itemsInTheCart.keys();
    while (shirtIDs.hasMoreElements()) {
      Integer shirtID = (Integer) shirtIDs.nextElement();
      Integer quantity = (Integer) itemsInTheCart.get(shirtID);

      try {
        // Find an associated TShirt entity bean
        ejava.ejb.entity.tshirt.CMRTShirtLocal beanShirt =
          tshirtHome.findByPrimaryKey(shirtID);
        // Create and initialize a Serializable TShirt object
        //    with data from TShirt entity bean.
        TShirt clientShirt =
          new TShirt(
            beanShirt.getShirtID().intValue(),
            beanShirt.getShirtSize(),
            beanShirt.getColor(),
            beanShirt.getDesignFront(),
            beanShirt.getDesignBack(),
            beanShirt.getPictureFront(),
            beanShirt.getPictureBack(),
            beanShirt.getUnitPrice());
        clientShirt.setQuantity(quantity.intValue());
        clientShirt.setTotalPrice(
          quantity.intValue() * beanShirt.getUnitPrice());
        // Add Serializable TShirt object to return vector
        returnItemsInTheCart.addElement(clientShirt);
      } catch (FinderException fe) {
        fe.printStackTrace();
        throw new EJBException(fe.getMessage());
      }
    }

    // Return Vector of Serializable TShirt objects
    return returnItemsInTheCart;
  }

  /**
   * Set default order shipping information and
   * create an order based on customer ID
   * and cart information.
   */
  private Order fillDefaultOrderInformation(
    int customerID,
    String shipInstruction,
    double shipWeight,
    double shipCharge)
    throws FinderException {
    Order order = new Order();
    // Default date to order is created
    order.setOrderDate(new Date(System.currentTimeMillis()));
    // Default date to be paid
    order.setPaidDate(
      new Date(System.currentTimeMillis() + THIRTY_DAYS));
    // Default date to be shipped
    order.setShippingInstructions(shipInstruction);
    order.setShipWeight(shipWeight);
    order.setShipCharge(shipCharge);
    order.setShipDate(new Date(System.currentTimeMillis() + FIVE_DAYS));
    // Set customer info for order being created
    Customer customer = getCustomerInfo(customerID);
    order.setCustomer(customer);
    order.setBillingAddress(customer.getAddress());
    order.setShippingAddress(customer.getAddress());
    // Retrieve items from shopping cart session info
    Vector itemsInTheCart = getSelectedItemsInTheCart();
    // Create items in the cart as array of items
    TShirt[] items = new TShirt[itemsInTheCart.size()];
    for (int i = 0; i < itemsInTheCart.size(); i++) {
      items[i] = (TShirt) itemsInTheCart.elementAt(i);
    }
    // Set items onto order
    order.setItems(items);
    return order;
  }

  /** Get Customer object based on customer ID */
  private Customer getCustomerInfo(int customerID)
    throws FinderException {
    CMRCustomerLocal customerLocal =
      customerHome.findByPrimaryKey(new Integer(customerID));
    Customer customer = fillReturnCustomer(customerLocal);
    customer.setCustomerID("" + customerID);
    return customer;
  }

  /** Get Customer object from local EJB interface */
  private Customer fillReturnCustomer(CMRCustomerLocal customerLocal) {
    Address address =
      new Address(
        customerLocal.getAddress_1(),
        customerLocal.getAddress_2(),
        customerLocal.getCity(),
        customerLocal.getState(),
        customerLocal.getZipCode());
    Customer customer =
      new Customer(
        customerLocal.getFirstName(),
        customerLocal.getLastName(),
        customerLocal.getMiddleName(),
        address,
        customerLocal.getPhone(),
        customerLocal.getEmail(),
        null);
    return customer;
  }

  /**
   * Checkout based on cart information
   */
  public Order checkOut(Order orderInformation) {
    try {
      // Get Vector of items from the shopping cart
      Vector itemsInTheCart = getSelectedItemsInTheCart();
      // Create items in the cart as array of items
      TShirt[] items = new TShirt[itemsInTheCart.size()];
      for (int i = 0; i < itemsInTheCart.size(); i++) {
        items[i] = (TShirt) itemsInTheCart.elementAt(i);
      }
      // Set items from cart into order information
      orderInformation.setItems(items);
      // Return new order as created with order manager
      return createAnOrder(orderInformation);
    } catch (CreateException ce) {
      throw new EJBException(ce.getMessage());
    }
  }

  /**
   * Checkout based on cart information. Set default shipping
   */
  public Order checkOut(int customerID) {
    try {
      // Now create and set default order information...
      // Default shipping instruction
      String shippingInstruct = "Leave package at door.";
      // Default max shipping weight
      double shipWeight = 5.0;
      // Default shipping charge
      double shipCharge = 1.95;
      // Default max paid date
      java.sql.Date paidDate =
        new java.sql.Date(System.currentTimeMillis() + THIRTY_DAYS);
      // Get default order information
      Order defaultOrder =
        fillDefaultOrderInformation(
          customerID,
          shippingInstruct,
          shipWeight,
          shipCharge);
      // Return new order as created with order manager
      return createAnOrder(defaultOrder);
    } catch (CreateException ce) {
      throw new EJBException(ce.getMessage());
    } catch (FinderException fe) {
      throw new EJBException(fe.getMessage());
    }

  }

  /**
   * Create an order using order manager and get order ID
   */
  private Order createAnOrder(Order order) throws CreateException {
    OrderManagerLocal orderManager = this.orderManagerHome.create();
    Integer newOrderID = orderManager.createNewOrder(order);
    order.setOrderID(newOrderID.intValue());
    return order;
  }
}
