package ejava.ejb.timer;

import java.rmi.RemoteException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.Queue;
import javax.jms.QueueSender;
import javax.jms.Session;
import java.util.Date;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import ejava.servlets.Address;
import ejava.servlets.Customer;
import ejava.servlets.Item;
import ejava.servlets.TShirt;
import ejava.servlets.Order;

public class MessageTimerClient {
  // Queue interfaces
	private QueueConnection connection;
	private QueueSession requesterSession;
	private QueueSender requester;
	private Queue requesterQueue;
	private long resetDuration = 50;

	/* Constructor. Establish JMS Sender */
	public MessageTimerClient(
		String connectionFactoryName,
		String queueName)
		throws Exception {
		// establish connection to Naming Service
		InitialContext context = new InitialContext();
		// Lookup for a JMS ConnectionFactory
		QueueConnectionFactory queueConnectionFactory =
			(QueueConnectionFactory) context.lookup(connectionFactoryName);
		// Create a JMS connection of type QueueConnection
		QueueConnection connection =
			queueConnectionFactory.createQueueConnection();
		// Create a JMS session object of type Queue
		QueueSession requesterSession =
			connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		// Lookup a JMS queue from the Naming Service
		Queue requesterQueue = (Queue) context.lookup(queueName);
		// Create a JMS Sender and receiver
		QueueSender requester =
			requesterSession.createSender(requesterQueue);
		// Intialize the Chat application

		set(connection, requesterSession, requester, requesterQueue);
		// Start the JMS connection; allows messages to be delivered
		connection.start();

	}

	/* Initializes the current instance variables */
	private void set(
		QueueConnection connection,
		QueueSession senderSession,
		QueueSender sender,
		Queue senderQueue) {
		this.connection = connection;
		this.requesterSession = senderSession;
		this.requester = sender;
		this.requesterQueue = senderQueue;
	}

	/** Send a timer reset message */
	protected void writeMessage() throws JMSException {
		// Create a Message
		TextMessage message = requesterSession.createTextMessage();
		// Create a message body
		message.setText("" + resetDuration);
		System.out.println("Sending Reset Duration :" + resetDuration);
		// Send the message using a QueueSender (requester variable)
		this.requester.send(message);
	}
  
	/* Run the MessageTimer Client */
	public static void main(String[] args) throws Exception {
		if (args.length != 2)
			System.out.println("Queue missing");
		MessageTimerClient sender =
			new MessageTimerClient(args[0], args[1]);
		// Read from command line for pause
		BufferedReader commandLineReader =
			new java.io.BufferedReader(new InputStreamReader(System.in));
		sender.writeMessage();
	}
}
