package ejava.ejb.timer;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.TimedObject;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import java.util.Collection;
import java.util.Iterator;
import java.util.Calendar;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;

import ejava.ejb.entity.customer.CustomerLocal;
import ejava.ejb.entity.customer.CustomerLocalHome;
import ejava.ejb.EJBHelper;

public class SessionTimerEJBean implements SessionBean, TimedObject {
	// Local attributes
	private SessionContext sessionContext;
	private CustomerLocalHome customerHome;
	private static final int N_DAYS = 30;

	// Standard session bean interfaces
	public void setSessionContext(SessionContext aCtx)
		throws RemoteException {
		sessionContext = aCtx;

		// lookup customer home
		try {
			lookupCustomerHome();
		} catch (NamingException ne) {
			ne.printStackTrace();
		}
	}
	public void ejbCreate() { /* no removal logic */
	}
	public void ejbRemove() { /* no removal logic */
	}
	public void ejbPassivate() { /* no state persistence */
	}
	public void ejbActivate() { /* no state activation */
	}

	/** 
	 * Timed object method implementation.
	 * Prints out email delivery message info and resets
	 * the timer duration.
	 */
	public void ejbTimeout(Timer timer) {
		System.out.println("SessionTimerEJBean: ejbTimeout  Called! ");
		System.out.println(
			" I will start sending email now!:" + System.currentTimeMillis());
		try {
			// Print out email to Every customer.
			sendEmailToEveryCustomer();
			// Reset timer to default Duration
			resetTimerDuration(getDuration());
		} catch (FinderException fe) {
			fe.printStackTrace();
		}
	}

	/**
	 * Implements the public exposed EJB method.
	 * This gets the timer service and creates a timer
	 * using the duration passed into the method.
	 */
	public void resetTimerDuration(long duration) {
		System.out.println("SessionTimerEJBean: start resetTimer ");
		TimerService timerService = sessionContext.getTimerService();
		System.out.println("Current Timer Service is : " + timerService);
		Timer timer = timerService.createTimer(duration, "Email Timer");
		System.out.println("New Timer is :" + timer);
	}

	/**
	 * Implements the public exposed EJB method.
	 * First cancels all existing timers.
	 * It then gets the timer service and creates a timer
	 * using the duration passed into the method.
	 */
	public void resetTimer(long duration) {
		System.out.println("SessionTimerEJBean: start createTimer ");
		// Cancel Existing Timers
		cancelExistingTimers();
		// creae new timer with new duration
		resetTimerDuration(duration);
	}


	/**
	 * Gets the timers from the timer service and
	 * cancels every one of them.
	 */
	private void cancelExistingTimers() {
		TimerService timerService = sessionContext.getTimerService();
		Iterator timers = timerService.getTimers().iterator();
		while (timers.hasNext()) {
			Timer timer = (Timer) timers.next();
			timer.cancel();
		}
	}

	/** 
	 * Get email for each customer in the database.
	 * Indicate that are sending them an email.
	 * For heaven's sake, we're not actually going to
	 * do that for this example!!!!
	 * Thus...we just print a message here.
	 */
	private void sendEmailToEveryCustomer() throws FinderException {
		Iterator iterator = customerHome.findAll().iterator();
		while (iterator.hasNext()) {
			CustomerLocal customer = (CustomerLocal) iterator.next();
			String emailAddress = customer.getEmail();
			System.out.println(
				"Sending Email to :"
					+ customer.getFirstName()
					+ ","
					+ customer.getLastName());
		}
	}

	/** Lookup Customer Home object */
	private void lookupCustomerHome() throws NamingException {
		if (customerHome == null) {
			InitialContext context = new InitialContext();
			customerHome =
				(CustomerLocalHome) context.lookup(EJBHelper.CUSTOMER_REFNAME);
			context.close();
		}
	}
	
	/** Get duration of 30 days from now */
	private long getDuration(){
		// Set a default timer duration as 30 days from now
		Calendar time = Calendar.getInstance();
		time.add(Calendar.DATE, N_DAYS);
		return time.getTimeInMillis();		
	}
}
