package ejava.ejb.transactions;

import javax.ejb.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;
import javax.transaction.*;

public class MySessionEJBean implements SessionBean {
	private SessionContext ejbContext;
	private DataSource ordersDatabase = null;
	private DataSource suppliersDatabase = null;
	private Connection ordersConnection = null;
	private Connection suppliersConnection = null;

	public void setSessionContext(SessionContext aCtx) {
		ejbContext = aCtx;
	}

	protected UserTransaction getJNDIUserTramsaction() {
		UserTransaction transaction = null;

		try {
			// Acquire handle to JNDI context from container environment
			Context jndiCtx = new InitialContext();

			// Lookup UserTransaction from container environment
			transaction =
				(UserTransaction) jndiCtx.lookup("java:comp/UserTransaction");
		} catch (NamingException e) {
			// handle naming exception here
		}

		return transaction;
	}

	// Application-specific interfaces
	public void resolveOrders(String supplierName) {

		// Get UserTransaction
		UserTransaction transaction = ejbContext.getUserTransaction();

		try {
			// Get context and data sources and connections
			InitialContext jndiCtx = new InitialContext();
			DataSource ordersDatabase =
				(DataSource) jndiCtx.lookup("java:comp/env/jdbc/Orders");
			DataSource suppliersDatabase =
				(DataSource) jndiCtx.lookup("java:comp/env/jdbc/Suppliers");
			Connection ordersConnection = ordersDatabase.getConnection();
			Connection suppliersConnection =
				suppliersDatabase.getConnection();

			// Possibly set isolation level for a connection
			suppliersConnection.setTransactionIsolation(
				Connection.TRANSACTION_READ_COMMITTED);

			// Begin transaction
			transaction.begin();

			// Do work...
			// 1) Create JDBC statements from connections.
			// 2) Do a bunch of JDBC update-related work on statements.

			// If error occurs while doing work...
			transaction.rollback();

			// Else if all is OK when done...end Transaction
			transaction.commit();

			// Close any connection and statements
		} catch (Exception e) {
			// Error occurred while doing work...
			try {
				transaction.rollback();
			} catch (SystemException se) {
				se.printStackTrace();
			}
			e.printStackTrace();
		}
	}

	public void placeOrder(String orderID, Object orderInfo) {
    // Get UserTransaction
    UserTransaction transaction = ejbContext.getUserTransaction();

		try {
			// Get context and data sources and connections
			InitialContext jndiCtx = new InitialContext();
			ordersDatabase =
				(DataSource) jndiCtx.lookup("java:comp/env/jdbc/Orders");
			ordersConnection = ordersDatabase.getConnection();

			// Begin transaction
			transaction.begin();

			// Do work...
			// 1) Create JDBC statements from connections.
			// 2) Update order database.
    } catch (Exception e) {
      // Error occurred while doing work...
      try {
        transaction.rollback();
      } catch (SystemException se) {
        se.printStackTrace();
      }
      e.printStackTrace();
    }
	}

	public void notifySupplier(String supplierName, String orderID) {
    // Get UserTransaction
    UserTransaction transaction = ejbContext.getUserTransaction();

		try {
			// Get context and data sources and connections
			InitialContext jndiCtx = new InitialContext();
			suppliersDatabase =
				(DataSource) jndiCtx.lookup("java:comp/env/jdbc/Suppliers");
			suppliersConnection = suppliersDatabase.getConnection();

			// Possibly set isolation level for a connection
			suppliersConnection.setTransactionIsolation(
				Connection.TRANSACTION_READ_COMMITTED);

			// Do work...
			// 1) Create JDBC statements from connections.
			// 2) Do a bunch of JDBC update-related work on
			//     order and suppliers database statements.

			// If error occurs while doing work...
			transaction.rollback();

			// Else if all is OK when done...end Transaction
			transaction.commit();

			// Close any connection and statements
    } catch (Exception e) {
      // Error occurred while doing work...
      try {
        transaction.rollback();
      } catch (SystemException se) {
        se.printStackTrace();
      }
      e.printStackTrace();
    }
	}

	// Standard interfaces
	public void ejbCreate() { /* implement any creation logic */
	}
	public void ejbRemove() { /* implement any removal logic */
	}
	public void ejbPassivate() { /* implement any state persistence */
	}
	public void ejbActivate() { /* implement any state activation */
	}
}