package ejava.jaxp;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXParseException;

import java.io.File;

/**
 * An example document handler
 */
public class SampleContentHandler  implements ContentHandler
{

  /**
   * Receives the character data for each element
   */
  public void characters(char[] value, int start, int length)
      throws SAXException
  {
    String newValue = new String(value);
    System.out.println(" Present Char Chunk :" +
      newValue.substring(start,start+length) );
  }

  /**
   * Receives notification of end of document
   */
  public void endDocument()
    throws SAXException
  {
    System.out.println("End of Document ");
  }

  /**
   * Receives information about end of an element
   */
  public void endElement(String namespaceURI, 
                         String localName, String qName)
    throws SAXException
  {
    System.out.println(" End of Element :");
    System.out.println("   namespaceURI = " + namespaceURI);
    System.out.println("   localName    = " + localName);
    System.out.println("   qName        = " + qName);
  }

  /**
   * Ends the scope of a prefix-URI Namespace mapping.
   * Called after the corresponding endElement.
   */
  public void endPrefixMapping(String prefix) throws SAXException
  {
    System.out.println(" End prefix:");
    System.out.println("    prefix = " + prefix);
  }

  /**
   * Receives notification of ignorable whitespace in element content.
   */
  public void ignorableWhitespace(char[] value, int start , int length)
    throws SAXException
  {
    String newValue = new String(value);
    if(length > 0){
     System.out.println(" ignorable Value is :"
                        + newValue.substring(start,start+length-1));
    }
  }

  /**
   * The XMLReader will invoke this method once for each processing
   * instruction found
  */
  public void processingInstruction(String target, String data)
    throws SAXException
  {
    System.out.println("Processing Instruction Target :"+target);
    System.out.println("Processing Instruction data :"+data);
  }

  /**
   * Receives a handle to the XMLReader's Locator object
   */
  public void setDocumentLocator(Locator locator)
  {
    System.out.println("Received locator");
  }

  /**
   * Receives notification of a skipped entity
   */
  public void skippedEntity(String name) throws SAXException
  {
      System.out.println("Skipped entity = " + name);
  }

  /**
   *  Receives the startDocument notification
   */
  public void startDocument()
    throws SAXException
  {
    System.out.println(" The parser started parsing document :");
  }
  
  /**
   * Info about the new element being parsed, including
   * namespace, local and qualified names of the element,
   * and the list of attributes.
   */
  public void startElement(String namespaceURI, String localName, 
                           String qName, Attributes attributes)
    throws SAXException
  {
    System.out.println(" Start of Element :");
    System.out.println("     namespaceURI = " + namespaceURI);
    System.out.println("     localName    = " + localName);
    System.out.println("     qName        = " + qName);

    int totalAttributes = attributes.getLength();
    for(int i = 0; i < totalAttributes; i++){
      String name = attributes.getLocalName(i);
      String type = attributes.getType(i);
      String value = attributes.getValue(i);
      System.out.println(" Attribute Name :" + name +
        " Type : "+ type +" Value :" +value);
    }
  }

  /**
   * Begins the scope of a prefix-URI Namespace mapping.
   * Called before the corresponding startElement.
   */
  public void startPrefixMapping(String prefix, String uri)
     throws SAXException
  {
    System.out.println(" Start prefix:");
    System.out.println("      prefix = " + prefix);
    System.out.println("      uri    = " + uri);
  }
}
