package ejava.jdbc.advanced;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.SQLException;
import java.lang.ClassNotFoundException;
import java.util.prefs.Preferences;
import java.util.StringTokenizer;

/**
 This class makes a connection to the database using configuration
 information provided in preferences.
 */
public class ConnectToDatabase
{
  private String driverClass; // Driver class name
  private String dbURL; // Database URL
  private Connection connection; // A database connection object


  public ConnectToDatabase()
  {
  }

  /**
   Checks to see if the DRIVER_CLASS, DATABASE_URL, username,
   password and subproperites from preferences object.
   Makes the connection to the database if the connection is
   not made yet.
   */
  public Connection getConnection()
    throws ClassNotFoundException, SQLException, Exception
  {
    if(connection != null){
      return this.connection;
    }
    Preferences preferences =
      Preferences.userNodeForPackage(ConnectToDatabase.class);
    // Get connection properties from preferences
    driverClass = preferences.get("DRIVER_CLASS", null);
    dbURL   = preferences.get("DATABASE_URL",null);
    String userName = preferences.get("USER_NAME",null);
    String password = preferences.get("PASSWORD",null);

    if(driverClass == null || dbURL == null){
      throw new Exception("Driver Class or Driver"+
                            " URL should not be null");
    }

    System.out.println("Registering driver : "+driverClass);
    // Load driver
    Class.forName(driverClass);

    // Create connection in one of two ways depending on whether or
    //not a user name, and password were in preferences
    if( (userName == null ) && (password == null)){
      System.out.println("Connecting to Database :"+dbURL);
      connection = DriverManager.getConnection(dbURL);
    }
    else{
      System.out.println("Connecting to DataBase :"+dbURL+
            " UserName :"+userName +"Password :"+password);
      connection =
          DriverManager.getConnection(dbURL, userName, password);
    }

    return connection;
  }
 
  
  
  public static void main(String[] args)
  {
    Preferences preferences =
      Preferences.userNodeForPackage(
      ejava.jdbc.advanced.ConnectToDatabase.class);
    ConnectToDatabase connectionToDatabase = new ConnectToDatabase();

    try{
      Connection connection = connectionToDatabase.getConnection();
    }
    catch(ClassNotFoundException classNotFoundException){
      String driverClass = preferences.get("DRIVER_CLASS",null);
      System.out.println( "Driver Class :" +driverClass
                           + "Exception :"+classNotFoundException);
      System.exit(0);
    }
    catch(SQLException sqlExcetpion){
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println( "Failed to connect database :" +dbURL
                           + "Exception :"+sqlExcetpion);
      System.exit(0);
    }
    catch(Exception exception){
      System.out.println( "Failed to connect database :"
                           + "Exception :"+exception);
      System.exit(0);
    }

    String driverClass = preferences.get("DRIVER_CLASS",null);
    String dbURL = preferences.get("DATABASE_URL",null);
    System.out.println("Everything was ok. Database was connected");
    System.out.println("Driver :" +driverClass);
    System.out.println("Database URL :"+dbURL);
    System.exit(0);
  }
}


