
package ejava.jdbc.advanced;

import java.util.prefs.Preferences;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.lang.ClassNotFoundException;
import java.sql.PreparedStatement;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.ResultSetMetaData;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.sql.Struct;
import java.sql.Types;
import java.sql.Ref;
import java.sql.Date;
import java.util.Map;
import java.util.GregorianCalendar;


/**
 * Class
 *
 *
 * @author
 * @version %I%, %G%
 */
public class NewSQLTypeExample extends ConnectToDatabase
{
  private final static int LOB_DATA_BUFFER_SIZE = 1024;

  /**
   * Constructor
   *
   *
   * @see
   */
  public NewSQLTypeExample()
  {
    super();
  }

  /**
   * Method
   *
   *
   * @return
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  protected Statement getStatement()
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Statement statement = connection.createStatement();

    return statement;
  }

  /**
   * Method
   *
   *
   * @param sqlString
   *
   * @return
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  protected PreparedStatement getStatement(String sqlString)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlString);

    return preparedStatement;
  }

  /**
   * Execute statement to insert Blob and print execute result
   */
  public void retrieveBLOBtype(String queryString)
    throws SQLException, Exception
  {
    Statement statement = getStatement();
    ResultSet rs = statement.executeQuery(queryString);

    while (rs.next()) {
      Blob blob = rs.getBlob(1);
      InputStream inputStream = blob.getBinaryStream();

      // open the stream to read data from Blob reference
      long length = blob.length();

      // Write Blob to file
      byte[] x = new byte[LOB_DATA_BUFFER_SIZE];
      int lengthRead = 0;
      String blobFile = "blobExample1.gif";
      FileOutputStream outputStream = new FileOutputStream(blobFile);
      while ((lengthRead =  inputStream.read(x)) != -1) {
        outputStream.write(x,0, lengthRead);
      }
      inputStream.close();
      outputStream.close();

      // Display Blob
      System.out.println(" About to display Blob ");
      (new ImageReader()).render("blobExample1.gif");
    }
  }

  /**
   * Execute statement to insert Blob and print execute result
   */
  public void insertABLOBtype(String updateString)
    throws SQLException, Exception
  {
    PreparedStatement preparedStatement = getStatement(updateString);
    byte[] dataInFile = readFile("blobExample.gif");
    preparedStatement.setBytes(1,dataInFile);
    boolean result = preparedStatement.execute();
  }

  private byte[] readFile(String fileToRead)
    throws Exception
  {

    File f = new File(fileToRead);
    int lengthofFile = (int) f.length();
    byte[] dataInFile = new byte[lengthofFile];
    FileInputStream fin = new FileInputStream(f);
    fin.read(dataInFile);
    return dataInFile;
  }

  /**
   * Method
   *
   *
   * @param queryString
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void retrieveCLOBtype(String queryString)
    throws SQLException, Exception
  {
    Statement statement = getStatement();
    ResultSet rs = statement.executeQuery(queryString);

    for(int i = 0; rs.next(); ++i) {
      Clob c = rs.getClob(1);
      InputStream inputStream = c.getAsciiStream();

      // open the stream to read data from Clob reference
      long length = c.length();

      byte[] x = new byte[LOB_DATA_BUFFER_SIZE];

      // Create byte array to hold the clob data
      int lengthRead = 0;    // Number of bytes read

      String clobFile = "clobExample" + i + ".txt";
      FileOutputStream outputStream = new FileOutputStream(clobFile);

      while ((lengthRead = inputStream.read(x)) != -1) {
        outputStream.write(x);
      }

      inputStream.close();
      outputStream.close();
    }
  }

  /**
   * Method
   *
   *
   * @param updateString
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void insertACLOBtype(String updateString)
    throws SQLException, Exception
  {
    PreparedStatement preparedStatement = getStatement(updateString);
    byte[] dataInFile = readFile("clobExample.txt");
    String data = new String(dataInFile);
    preparedStatement.setString(1,data);
    boolean result = preparedStatement.execute();
  }

  /**
   * Method
   *
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void mapCardClassToSQLTypes()
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Map map = connection.getTypeMap();

    map.put("CH06TYPES.CHARGE_CARD_TYPE",
            Class.forName("ejava.jdbc.advanced.ChargeCard"));
  }

  /**
   * Method
   *
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void insertCardTypeData()
    throws SQLException, Exception
  {
    String insertString =
      "INSERT INTO CH06TYPES.CREDIT VALUES( 200, 2002 , "+
      "'EMPTY_CLOB()', ?) ";
    Date expirationDate = Date.valueOf("2004-10-10");
    ChargeCard chargeCard =
      new ChargeCard("CH06TYPES.CHARGE_CARD_TYPE", "CREDIT", "VISA",
                     "123456789012", expirationDate,
                     "123456789012");
    PreparedStatement preparedStatement = getStatement(insertString);
    preparedStatement.setObject(1, chargeCard, Types.STRUCT);
    boolean result = preparedStatement.execute();
  }

  /**
   * Method
   *
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void retrieveCardTypeData()
    throws SQLException, Exception
  {
    String retrieveData =
      "SELECT * FROM CH06TYPES.CREDIT WHERE"
      + " CUSTOMER_ID_FK =101 AND ORDER_ID_FK = 1001 ";
    Statement statement = getStatement();
    ResultSet rs = statement.executeQuery(retrieveData);

    while (rs.next()) {
      ChargeCard c = (ChargeCard) rs.getObject(4);
      System.out.println(c);
    }
  }

  /**
   * Method
   *
   *
   * @param insertString
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void insertAStructTypeData(String insertString)
    throws SQLException, Exception
  {
    Statement statement = getStatement();
    int nRows = statement.executeUpdate(insertString);
    System.out.println(" Number rows written: " + nRows);
  }

  /**
   * Method
   *
   *
   * @param queryString
   *
   * @throws Exception
   * @throws SQLException
   *
   * @see
   */
  public void retrieveAStructTypeData(String queryString)
    throws SQLException, Exception
  {
    Statement statement = getStatement();
    ResultSet rs = statement.executeQuery(queryString);
    ResultSetMetaData rsMetaData = rs.getMetaData();

    while (rs.next()) {
      for (int i = 1 ; i <= rsMetaData.getColumnCount(); i++) {
        int type = rsMetaData.getColumnType(i);

        switch (type) {

          case Types.STRUCT:
            System.out.println("Struct Type Value is :");
            Struct s = (Struct) rs.getObject(i);
            String structTypeName = s.getSQLTypeName();
            Object[] contents = s.getAttributes();
            for (int j = 0; j < contents.length; j++) {
              System.out.println(contents[j]);
            }

            break;
          default:
            System.out.println("Non Struct Type Value :"+
                                 rs.getObject(i));
            break;
        }
      }
    }
  }
  /**
   * Method
   *
   *
   * @param args
   *
   * @see
   */
  public static void main(String[] args)
  {
    Preferences preferences = Preferences.userNodeForPackage(
                        ejava.jdbc.advanced.ConnectToDatabase.class);

    NewSQLTypeExample newSqlTypeExample = new NewSQLTypeExample();

    try {

      System.out.println(" Blob Type Insert Example ");
      String blobTypeInsert =
        preferences.get("INSERT_A_BLOB_TYPE_STATEMENT",null);
      newSqlTypeExample.insertABLOBtype(blobTypeInsert);

      System.out.println(" Blob Type Retrieval Example ");
      String blobTypeRetrieve =
        preferences.get("RETRIEVE_A_BLOB_TYPE_STATEMENT",null);
      newSqlTypeExample.retrieveBLOBtype(blobTypeRetrieve);

      System.out.println(" Struct Type Insert Example ");
      String structTypeInsert =
        preferences.get("INSERT_STRUCT_TYPE_STATEMENT",null);
      newSqlTypeExample.insertAStructTypeData(structTypeInsert);

      System.out.println(" Struct Type Retrieval Example ");
      String structTypeRetrieve =
        preferences.get("RETRIEVE_STRUCT_TYPE_STATEMENT",null);
      newSqlTypeExample.retrieveAStructTypeData(structTypeRetrieve);

      System.out.println(" Clob Type Insert Example ");
      String clobTypeInsert =
        preferences.get("INSERT_A_CLOB_TYPE_STATEMENT",null);
      newSqlTypeExample.insertACLOBtype(clobTypeInsert);

      System.out.println(" Clob Type Retrieval Example ");
      String clobTypeRetrieve =
        preferences.get("RETRIEVE_A_CLOB_TYPE_STATEMENT",null);
      newSqlTypeExample.retrieveCLOBtype(clobTypeRetrieve);

      System.out.println(" ChargeCard Type Mapping Example");
      newSqlTypeExample.mapCardClassToSQLTypes();

      System.out.println(" ChargeCard Type Insertion Example ");
      newSqlTypeExample.insertCardTypeData();

      System.out.println(" ChargeCard Type Retrieval Example ");
      newSqlTypeExample.retrieveCardTypeData();

    }
    catch (ClassNotFoundException classNotFoundException) {
      String driverClass = preferences.get("DRIVER_CLASS",null);
      System.out.println("Driver Class :" + driverClass +
                           " Exception :" + classNotFoundException);
      System.exit(0);
    } catch (SQLException sqlExcetpion) {
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :" + dbURL
                           + " Exception :" + sqlExcetpion);
      System.exit(0);
    } catch (Exception exception) {
      System.out.println("Failed to connect db :" + " Exception :"
                           + exception);
      System.exit(0);
    }

    System.exit(0);
  }

}

