package ejava.jdbc.core;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.CallableStatement;
import java.sql.SQLException;
import java.lang.ClassNotFoundException;
import java.sql.Types;
import java.util.prefs.Preferences;

/**
 * Class  CallableStatementExample
 * @author
 * @version %I%, %G%
 */
public class CallableStatementExample extends ConnectToDatabase
{
	
	/**
	 * Constructor
	 *
	 */
	public CallableStatementExample()
	{
		super();
	}
	
	
	/**
	 * Method    getCallableStatement
	 * @param sqlStatement
	 * @return
	 * @throws Exception
	 * @throws SQLException
	 * @see
	 */
	protected CallableStatement getCallableStatement(String sqlStatement)
		throws SQLException, Exception
	{
		Connection connection = super.getConnection();
		CallableStatement statement =
			connection.prepareCall(sqlStatement);
		
		return statement;
	}
	
	/**
	 * Method
	 * @param sqlStatement
	 * @param resultSetType
	 * @param resultSetConcurrency
	 * @return
	 * @throws Exception
	 * @throws SQLException
	 * @see
	 */
	protected CallableStatement getCallableStatement(String sqlStatement,
		int resultSetType, int resultSetConcurrency)
		throws SQLException, Exception
	{
		Connection connection = super.getConnection();
		CallableStatement statement = connection.prepareCall(sqlStatement,
			resultSetType, resultSetConcurrency);
		
		return statement;
	}
	
	/**
	 * Method
	 *
	 *
	 * @throws Exception
	 * @throws SQLException
	 *
	 * @see
	 */
	public void executeGetTotalPriceFunction()
		throws SQLException, Exception
	{
		String sqlStatement =
			"begin ? := getTotalPriceForOrder( 1001);end;";
		CallableStatement callableStatement =
			getCallableStatement(sqlStatement);
		
		callableStatement.registerOutParameter(1, Types.FLOAT);
		callableStatement.execute();
		System.out.println("Total price :" +
			callableStatement.getFloat(1));
	}
	
	/**
	 * Method
	 *
	 *
	 * @throws Exception
	 * @throws SQLException
	 *
	 * @see
	 */
	public void executeGetCustomerPhoneNumberFunction()
		throws SQLException, Exception
	{
		String sqlStatement =
			"begin ? := getCustomerPhoneNumber(101); end;";
		CallableStatement callableStatement =
			getCallableStatement(sqlStatement);
		
		callableStatement.registerOutParameter(1, Types.VARCHAR);
		callableStatement.execute();
		System.out.println("Phone #  :" + callableStatement.getString(1)
									 + " for Customer_id 101");
	}
	
	/**
	 * Method
	 *
	 *
	 * @throws Exception
	 * @throws SQLException
	 *
	 * @see
	 */
	public void executeTotalOrdersFortheCustomer()
		throws SQLException, Exception
	{
		String sqlStatement = "begin   getTotalOrders(104, ? ); end;";
		CallableStatement callableStatement =
			getCallableStatement(sqlStatement);
		
		callableStatement.registerOutParameter(1, Types.INTEGER);
		callableStatement.execute();
		System.out.println("Total Orders #  :" +
			callableStatement.getInt(1) + " for Customer_id 104");
	}
	
	/**
	 * Method
	 *
	 *
	 * @param args
	 *
	 * @see
	 */
	public static void main(String[] args)
	{
		Preferences preferences =
			Preferences.userNodeForPackage(
			ejava.jdbc.core.ConnectToDatabase.class);
		
		CallableStatementExample callStExample =
			new CallableStatementExample();
		
		try {
			callStExample.executeGetCustomerPhoneNumberFunction();
			callStExample.executeGetTotalPriceFunction();
			callStExample.executeTotalOrdersFortheCustomer();
		} catch (ClassNotFoundException classNotFoundException) {
			String driverClass = preferences.get("DRIVER_CLASS",null);
			
			System.out.println("Driver Class :" + driverClass +
				" Exception :" + classNotFoundException);
			System.exit(0);
		} catch (SQLException sqlExcetpion) {
			String dbURL = preferences.get("DATABASE_URL",null);
			
			System.out.println("Failed to connect db :" + dbURL
										 + "Exception :" + sqlExcetpion);
			System.exit(0);
		} catch (Exception exception) {
			System.out.println("Failed to connect db :" + " Exception :"
										 + exception);
			System.exit(0);
		}
		
		System.exit(0);
	}
	
}

