package ejava.jdbc.core;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import java.util.Vector;
import java.sql.ResultSet;
import java.util.prefs.Preferences;

/**
 This example can work only if the db Driver supports DatabaseMetaData.
 */
public class DatabaseMetaDataExample extends ConnectToDatabase
{
	private DatabaseMetaData dbMetaData; // Database MetaData Object
	
	public DatabaseMetaDataExample()
	{
		super();
	}
	
	
	/**
	 * The Connection object associated with the database returns
	 * meta data associated with the database.
	 */
	public void getDatabaseMetaData()
		throws SQLException, Exception
	{
		if(dbMetaData == null){
			Connection connection = super.getConnection();
			dbMetaData = connection.getMetaData();
		}
	}
	
	/**
	 Gets the Schema names contained in the database and uses the
	 ResultSetExample to parse the returned ResultSet containing
	 the single column describing the catalog name.
	 */
	public void printAvailableSchemas()
		throws SQLException, Exception
	{
		if(dbMetaData == null){
			getDatabaseMetaData();
		}
		
		ResultSet resultSet = dbMetaData.getSchemas();
		ResultSetExample.parseResultData(resultSet);
		resultSet.close();
	}
	
	/**
	 Gets the the tables available in a catalog.
	 */
	public  void printTablesInAllSchemas()
		throws SQLException, Exception
	{
		if(dbMetaData == null){
			getDatabaseMetaData();
		}
		
		ResultSet resultSet = dbMetaData.getSchemas();
		while(resultSet.next()){
			String schemaName = resultSet.getString(1);
			String catalog = null;
			String schemaPattern = schemaName;
			String tableNamePattern = null;
			String types[]  = {"TABLE", "VIEW"};
			ResultSet tableInformation =
				dbMetaData.getTables(catalog,schemaPattern,
				tableNamePattern, types);
			ResultSetExample.parseResultData(tableInformation);
		}
	}
	
	/**
	 Gets the the tables available in a Schema.
	 */
	public  void printTablesInASchema(String schemaName)
		throws SQLException, Exception
	{
		if(dbMetaData == null){
			getDatabaseMetaData();
		}
		
		String catalog = null;
		String schemaPattern = schemaName;
		String tableNamePattern = null;
		String types[]  = {"TABLE", "VIEW"};
		ResultSet tableInformation =
			dbMetaData.getTables(catalog,schemaPattern,
			tableNamePattern, types);
		ResultSetExample.parseResultData(tableInformation);
		
	}
	/**
	 Gets the primary keys in a Tables
	 */
	
	public  void printPrimaryKeysInATable(String schemaName,
		String tableName)
		throws SQLException, Exception
	{
		if(dbMetaData == null){
			getDatabaseMetaData();
		}
		
		String catalog = null;
		String schemaPattern = schemaName;
		ResultSet tableInformation
			= dbMetaData.getPrimaryKeys(catalog,schemaPattern,tableName);
		ResultSetExample.parseResultData(tableInformation);
	}
	
	public void printColumnsInATable(String schemaName, String tableName)
		throws SQLException , Exception {
		
		if(dbMetaData == null){
			getDatabaseMetaData();
		}
		
		String catalogName = null;
		String columnPattern = null;
		ResultSet resultSet =
			dbMetaData.getColumns(catalogName,schemaName,
			tableName, columnPattern);
		
		ResultSetExample.parseResultData(resultSet);
	}
	
	public static void main(String[] args)
	{
		
		Preferences preferences =
			Preferences.userNodeForPackage(
			ejava.jdbc.core.ConnectToDatabase.class);
		
		DatabaseMetaDataExample databaseMetaDataExample =
			new DatabaseMetaDataExample();
		try{
			Connection connection =
				databaseMetaDataExample.getConnection();
			// Induce analysis of meta data in various ways...
			System.out.println("Printing Available Schemas :");
			databaseMetaDataExample.printAvailableSchemas();
			System.out.println("Printing All Tables in all  Schemas :");
			databaseMetaDataExample.printTablesInAllSchemas();
			
			String schemaName = preferences.get("SCHEMA_NAME",null);
			System.out.println("Printing All Tables in Schema :"
										 +schemaName);
			databaseMetaDataExample.printTablesInASchema(schemaName);
			System.out.println("Printing All Columns in Table CUSTOMER ");
			databaseMetaDataExample.
				printColumnsInATable(schemaName, "CUSTOMER");
			System.out.println("Printing Primary Keys for TableCUSTOMER ");
			databaseMetaDataExample.
				printPrimaryKeysInATable(schemaName,"CUSTOMER");
		}
		catch(ClassNotFoundException classNotFoundException){
			String driverClass = preferences.get("DRIVER_CLASS",null);
			System.out.println( "Driver Class :" +driverClass +
				" Exception :"+classNotFoundException);
			System.exit(0);
		}
		catch(SQLException sqlExcetpion){
			String dbURL = preferences.get("DATABASE_URL",null);
			System.out.println( "SQLException with database db :" +dbURL+
				" Exception :"+sqlExcetpion);
			System.exit(0);
		}
		catch(Exception exception){
			System.out.println( "General exception :"+exception);
			System.exit(0);
		}
		System.exit(0);
	}
}
