package ejava.jdbc.core;

import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Connection;
import java.util.prefs.Preferences;

/**
 This exmple gives an overview of the StatementObject and how to use
 different frequently used Statement related methods.
 */
public class SimpleStatementExample extends ConnectToDatabase
{
	private Statement  statement; // Statement object
	
	public SimpleStatementExample()
	{
		super();
	}
	
	/**
	 If not connected to database...connect to database, create a
	 statement,	 and, return the Statement.
	 */
	protected void createStatement()
		throws SQLException, Exception
	{
		if(statement == null){
			Connection connection = super.getConnection();
			statement = connection.createStatement();
		}
	}
	
	/**
	 This method executes a statement and returns true or false. Any
	 type of Statement may be executed by this command.
	 */
	public boolean execute(String sqlString)
		throws SQLException, Exception
	{
		if(statement == null){
			createStatement();
		}
		
		boolean returnValue = statement.execute(sqlString);
		return returnValue;
	}
	
	/**
	 This method executes the statement and returns a ResultSet from
	 execution of the query. Only a SELECT Statemet is applicable here.
	 */
	public ResultSet executeQuery(String sqlString)
		throws SQLException, Exception
	{
		if(statement == null){
			createStatement();
		}
		
		ResultSet rs = statement.executeQuery(sqlString);
		return rs;
	}
	
	/**
	 This method executes update statements and returns a count of
	 the updates. Only a INSERTS, DELETES, and UPDATES are applicable
	 here. In addition, SQL statements that return nothing such as
	 SQL DDL statements can be executed.
	 */
	public int executeUpdate(String sqlString)
		throws SQLException, Exception
	{
		if(statement == null){
			createStatement();
		}
		
		int returnValue = statement.executeUpdate(sqlString);
		return returnValue;
	}
	
	/**
	 This method closes the Statement.
	 */
	public void closeStatement()
		throws SQLException
	{
		if(statement != null){
			statement.close();
			statement = null;
		}
	}
	
	public static void main(String[] args)
	{
		Preferences preferences =
			Preferences.userNodeForPackage(
			ejava.jdbc.core.ConnectToDatabase.class);
		
		SimpleStatementExample simpleStatementExample =
			new SimpleStatementExample();
		try{
			
			Connection connection 
				= simpleStatementExample.getConnection();
			
			// Get SQL statements from preferences and execute them...
			
			// Execute a SQL query two ways
			String sqlString 
				= preferences.get("SQL_QUERY_STATEMENT",null);			
			System.out.println("Executing SQL Query Statement :"
								+sqlString);
			boolean executionStatus =
				simpleStatementExample.execute(sqlString);		
			if(executionStatus == false){
				System.out.println("Failed to Execute The Statement :"
											+ sqlString);
			}
			ResultSet resultSet =
				simpleStatementExample.executeQuery(sqlString);

			// Execute a SQL update
			sqlString =
				preferences.get("SQL_UPDATE_STATEMENT",null);
			System.out.println("Executing SQL update Statement :"+
										 sqlString);
			int count = simpleStatementExample.executeUpdate(sqlString);
			System.out.println("Number of Updates :"+count);
			
			// Execute a SQL join
			String joinStatement =
				preferences.get("SQL_QUERY_STATEMENT_JOIN",null);
			System.out.println("Executing SQL join Statement :"+
										 joinStatement);
			resultSet 
				= simpleStatementExample.executeQuery(joinStatement);
			
			// Execute an outer join
			String outerJoinStatement =
				preferences.get("SQL_QUERY_STATEMENT_OUTER_JOIN",null);
			System.out.println("Executing SQL outer join Statement :"+
				outerJoinStatement);
			resultSet =
				simpleStatementExample.executeQuery(outerJoinStatement);
			
			// Execute a SQL LIKE clause
			String likeStatement =
				preferences.get("SQL_QUERY_STATEMENT_WITH_LIKE",null);
			System.out.println("Executing SQL like Statement :"+
				likeStatement);
			resultSet = simpleStatementExample.executeQuery(likeStatement);

			// Close the resources
			simpleStatementExample.closeStatement();
		}
		catch(ClassNotFoundException classNotFoundException){
			String driverClass = preferences.get("DRIVER_CLASS",null);
			System.out.println("Driver Class :" +driverClass+
				" Exception :"+classNotFoundException);
			System.exit(0);
		}
		catch(SQLException sqlExcetpion){
			String dbURL = preferences.get("DATABASE_URL",null);
			System.out.println( "SQL Exception from :" +dbURL+
			" Exception :"+sqlExcetpion);
			System.exit(0);
		}
		catch(Exception exception){
			System.out.println( "General exception :"+exception);
			System.exit(0);
		}
		
		System.exit(0);
	}
}
