package ejava.jms;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Session;
import javax.jms.MessageProducer;
import javax.jms.Destination;
import javax.jms.TextMessage;
import javax.jms.ObjectMessage;
import javax.jms.JMSException;
import javax.jms.Session;
import java.sql.SQLException;
import java.io.Serializable;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.prefs.Preferences;
import java.util.Properties;
import java.util.Hashtable;

/**
  * This class gets orders from the database and
  * submits new orders on a given topic to the consumer.
  */
public class UnifySupplier
{
  private ConnectionFactory connectionFactory;
  private Connection connection;
  private Session session;
  private MessageProducer producer;
  private Destination destination;
  private Properties properties;


  public UnifySupplier(Context context)
    throws NamingException, JMSException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);

    String jmsFactoryName =
        preferences.get("JMS_FACTORY_FOR_TOPIC",null);
    // get ConnectionFactory
    connectionFactory = (ConnectionFactory)
      context.lookup(jmsFactoryName);
    // Get TopicConnection
    connection = connectionFactory.createConnection();
    // Get Session for the Connection
    session =
        connection.createSession(false,
          Session.AUTO_ACKNOWLEDGE);
    String topicName = preferences.get("TOPIC_NAME",null);
      // determine if topic by this name already exists
    destination = (Destination) context.lookup(topicName);

    producer = session.createProducer(destination);

  }
/**
  * Send a message to a  Topic.
  */
  public void send(Object newMessage)
       throws JMSException
  {
    System.out.println(" Sending :"+newMessage);
    if(newMessage instanceof String){
      TextMessage sendingMessage = session.createTextMessage();
      connection.start();  // start Sending
      sendingMessage.setText((String)newMessage); // set the Text message
      producer.send(sendingMessage); // send Message
    }else {
      ObjectMessage sendingMessage = session.createObjectMessage();
      connection.start();   // start Sending
      sendingMessage.setObject((OrderRequest)newMessage); // set Object Message
      producer.send(sendingMessage); // send message
    }
  }

/**
  * Close JMS Reletated Objects.
  */
  public void close()
       throws JMSException
  {
    producer.close();
    session.close();
    connection.close();
  }

/**
  * it returns the initial Context to the Topic Service
  */
  private static InitialContext getInitialContext()
       throws NamingException
  {
    System.out.println(" Getting Initial Context :");
   // Connect to naming Service
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    Hashtable env = new Hashtable();
    String jndiFactory = preferences.get("JNDI_FACTORY", null);
    String providerURL  = preferences.get("JNDI_PROVIDER_URL", null);
    env.put(Context.INITIAL_CONTEXT_FACTORY, jndiFactory);
    env.put(Context.PROVIDER_URL, providerURL);
    return new InitialContext(env);
  }

  /**
  * Create a new Topic Publisher and publish the Order to that Topic
  */
  public static void createATopicAndPublishOrder(OrderRequest orderRequest)
  {
    try{
      InitialContext context = getInitialContext();
      UnifySupplier supplier =
        new UnifySupplier(context);
      supplier.send(orderRequest);
      supplier.close();
      context.close();
    }
    catch(JMSException jmsException){
      System.out.println("Error :"+jmsException);
      jmsException.printStackTrace();
    }
    catch(NamingException namingException){
      System.out.println("Error :"+namingException);
      namingException.printStackTrace();
    }
  }


  public static void main(String[] args)
       throws Exception
  {
    try{
      // Get The Orders if any
      Hashtable newOrders = OrderManager.getNewOrders();
      Enumeration keys = newOrders.keys();
      // For each order, create order request and publish it
      while(keys.hasMoreElements()){
        String  key = (String)keys.nextElement();
        OrderRequest orderRequest =
          (OrderRequest)newOrders.get(key);
        createATopicAndPublishOrder(orderRequest);
      }

    }
    catch(SQLException sqlException){
      System.out.println("Error :"+sqlException);
      sqlException.printStackTrace();
    }

  }
}










