package ejava.jndi.directory;

import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;
import javax.naming.directory.InitialDirContext;
import javax.naming.Context;
import javax.naming.NameNotFoundException;
import javax.naming.directory.DirContext;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;

/**
  * implements sample NIS Client to do
  * major operations on a NIS type naming Service.
  */
public class SampleNISExample
{
  DirContext context = null;
  Properties properties = new Properties();
  
  public SampleNISExample(Properties p)
  {
    super();
    properties = p;
  }

  /**
   * Constructor
   * Naming Client which is initialized with a jndi.properties file.
   * Uncomment appropriate lines for each service provider in
   * jndi.properties.
   *
   * @param propertiesFileName
   *
   * @see
   */
  public SampleNISExample(String propertiesFileName)
    throws IOException
  {
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);
    // Now load the file input stream into the JNDI environment
    // properties.
    // This of course assumes that the file input stream is in a valid
    // Properties format.
    properties.load(propertiesFileStream);
  }
  
  public String getTargetName(){
    return properties.getProperty("nis.target.name", "");
  }

   /**
   * Method
   *
   *
   * @return
   *
   * @throws NamingException
   *
   * @see
   */
  public DirContext getInitialContext()
    throws NamingException
  {
    // Create initial context if it is not already created
    if (context == null) {
      System.out.println("Properties :" + properties);
        // Read in a set of default paramters.
      context = new InitialDirContext(properties);
      System.out.println("Context is :" + context);
    }
    return context;
  }


  public void listContents(String target)
  {
    try{
      DirContext context = getInitialContext();
      NamingEnumeration namesList =  context.list(target);
      if(namesList == null){
        System.out.println(target + " contains empty names ");
      }
      else{
        while(namesList.hasMore()){
          System.out.println(namesList.next());
        }
      }
    }
    catch(Exception exception){
      System.out.println("Error :" +exception);
      exception.printStackTrace();
      System.exit(0);
    }


  }

  /**
   * Method
   *
   * Main Test Driver: java ejav.jndi.SimpleLookup
   * @param args
   *
   * @see
   */
  public static void main(String[] args)
  {

    if(args.length  == 0){
      System.out.println(" You should provide properties file :"+
        "java ejava.jndi.directory.SampleNISExample");
      System.exit(0);

    }

    // Set name of properties file to use in initializing the simple
    // lookup.
    String propertiesFileName = "jndi.Properties";
    //String propertiesFileName = "jndi.Properties";

    // Instantiate an instance of this NamingClient
    SampleNISExample sampleSearchExample = null;

    try {
        sampleSearchExample =
          new SampleNISExample(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }

    // Demonstrate lookup of a bound object name
    String name = sampleSearchExample.getTargetName();

    System.out.println("Going to Lookup " + name);
    sampleSearchExample.listContents(name);


  }
}
