package ejava.jndi.directory.dsml;


import org.w3c.dom.Element;
import com.sun.jndi.dsmlv2.BatchRequest;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.Attribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.ModificationItem;
import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;
import javax.naming.NamingEnumeration;
import javax.naming.directory.SearchResult;
import java.util.Enumeration;
import javax.naming.directory.SearchControls;
import javax.naming.event.EventDirContext;
import javax.naming.event.EventContext;
import java.net.URL;
import java.net.URLConnection;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.FileInputStream;
import java.util.Properties;

/**
  * This example demonstrates main operations on a Directory Service
  *  Using com.sun.jndi.dsmlv2.request.DsmlRequestCtxFactory
  *  Because it is request provider we can use this class
  * to generate DSML Request (BatchRequest) only.
  * This example method generateDSMLRequestAsAMemoryObject()
  * generates the request as a DOM Object in memory (BatchRequest).
  * This example mehtod generateDSMLRequestAsAMemoryObject()
  * generates the request as a SOAP Request's SOAP Envelope to be send
  * the request as a Object in memory (BatchRequest).
 */

public class GenerateDSMLRequest
{
  DirContext context = null;
  Properties properties = null;

  public GenerateDSMLRequest(Properties properties)
  {
    super();
    this.properties = properties;
  }

  /**
   * Constructor
   * Naming Client which is initialized with a jndi.properties file.
   * Uncomment appropriate lines for each service provider in
   * jndi.properties.
   *
   * @param propertiesFileName
   *
   * @see
   */
  public GenerateDSMLRequest(String propertiesFileName)
    throws IOException
  {
    properties = new Properties();
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);
    // Now load the file input stream into the JNDI environment
    // properties.
    // This of course assumes that the file input stream is in a valid
    // Properties format.
    properties.load(propertiesFileStream);
  }

  /**
   * Method
   * @return
   * @throws NamingException
   * @see
   */
  private DirContext getInitialContext()
    throws NamingException
  {
    // Create initial context if it is not already created
    if (context == null) {
        // Read in a set of default paramters.
      context = new InitialDirContext(properties);
      System.out.println("Context is :" + context);
    }
    return context;
  }
/**
  * Search The Directory Service using a Filter.
  */
  public void searchUsingFilterUsingSearch(String filter)
  {
    try{
      DirContext context = getInitialContext();
      SearchControls searchControls = new SearchControls();
      String[] resultAttributes = {"cn", "sn", "telephonenumber" };
      searchControls.setReturningAttributes(resultAttributes);
      // set the number of results
      searchControls.setCountLimit(10);

      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search("ou=People ", filter, searchControls);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * add a new Attribute to the Directory Service.
  */
  public void addANewAttribute()
  {
    try{
      DirContext context = getInitialContext();
      Attribute objclasses = new BasicAttribute("objectclass");
      objclasses.add("top");
      objclasses.add("person");
      objclasses.add("organizationalPerson");
      objclasses.add("inetOrgPerson");
      Attribute cn = new BasicAttribute("cn", "John Doe");
      Attribute sn = new BasicAttribute("sn", "Doe");
      Attribute telephonenumber = new BasicAttribute("telephonenumber",
        "777777777");
      Attribute givenNames = new BasicAttribute("givenname", "John");
      //Specify the dn we are adding
      String dn = "uid=jdoe, ou=People";
      Attributes organization = new BasicAttributes();
      organization.put(objclasses);
      organization.put(cn);
      organization.put(sn);
      organization.put(telephonenumber);
      organization.put(givenNames);
      context.createSubcontext(dn, organization);
    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * replace an Attribute in the Directory Service.
  */
  public void replaceAttributes()
  {
    try{
      DirContext context = getInitialContext();
      ModificationItem modificationItems[] = new ModificationItem[2];
      modificationItems[0] = new ModificationItem(
        DirContext.REPLACE_ATTRIBUTE,
        new BasicAttribute("sn", "Carter") );
      modificationItems[1] = new ModificationItem(
        DirContext.REPLACE_ATTRIBUTE,
        new BasicAttribute("telephonenumber", "111111111111") );

      context.modifyAttributes("uid=scarter, ou=People ",
        modificationItems);

      String filter = "(&(sn=Carter)(email=*))";

      searchUsingFilterUsingSearch(filter);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * remove attributes from the directory Service.
  */
  public void removeAttributes()
  {
    try{
      DirContext context = getInitialContext();
      ModificationItem modificationItems[] = new ModificationItem[1];

      modificationItems[0] = new ModificationItem(
        DirContext.REMOVE_ATTRIBUTE,
        new BasicAttribute("telephonenumber") );

      context.modifyAttributes("uid=jdoe, ou=People ",
        modificationItems);

      String filter = "(&(sn=Doe)(givenname=*))";

      searchUsingFilterUsingSearch(filter);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * Get Selected Attributes only during a search results.
  */
  public void getSelectedAttributesUsingGetAttributes()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter
      // and has the attribute givenname
      String resultAttributes[] = {"uid",
                            "givenname",
                            "sn",
                            "telephonenumber"
                            };
      // ignore name case
      // Search for objects with those matching attributes
      Attributes searchResults
        = context.getAttributes("uid=scarter,ou=People ",
                                resultAttributes);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * Get every Attribute for a search Results using GetAttributes method
  */
  public void getEveryAttributeUsingGetAttributes()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter
      // ignore name case, Search for objects with those matching attributes
      Attributes searchResults
        = context.getAttributes("uid=scarter, ou=People");

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * Get the every attribute using the search
  */
  public void getEveryAttributeUsingSearch()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter.
      // and has the attribute givenname
      BasicAttribute atr1 = new BasicAttribute("uid", "scarter");
      BasicAttribute atr2 = new BasicAttribute("givenname");
      Attributes matchAttrs = new BasicAttributes(true);
      matchAttrs.put(atr1);
      matchAttrs.put(atr2);
      // ignore name case
      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search("ou=People", matchAttrs);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }
/**
  * Get only selected attributes using serach methods.
  */
  public void getSelectedAttributesUsingSearch()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter
      // and has the attribute givenname
      BasicAttribute atr1 = new BasicAttribute("uid", "scarter");
      BasicAttribute atr2 = new BasicAttribute("givenname");
      Attributes matchAttrs = new BasicAttributes(true);
      matchAttrs.put(atr1);
      matchAttrs.put(atr2);

      String resultAttributes[] = {"uid",
                            "givenname",
                            "sn",
                            "telephonenumber"
                            };
      // ignore name case
      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search("ou=People", matchAttrs,resultAttributes);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }

  private void generateDSMLRequestAsAMemoryObject()
  {
    // Generate BatchRequest Object
    Element batchRequest =
         BatchRequest.toDomFromContext(context);
    // Generate Request as SOAP Envelope 
    Element soapEnvelope =
         BatchRequest.toDomSoapEnvelopeFromContext(context);
  }

  private void closeContext()
    throws NamingException
  {
    // unless we close the context , it won't write to file
    if(this.context != null){
         context.close();
         context = null;
    }
  }

  public static void main(String[] args)
    throws Exception
  {
     String propertiesFileName = "jndi.Properties";


    GenerateDSMLRequest sampleRequest = null;

    try {
        // Instantiate an instance of this NamingClient
        sampleRequest =
          new GenerateDSMLRequest(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }

    sampleRequest.getEveryAttributeUsingSearch();
    sampleRequest.getSelectedAttributesUsingSearch();

    // Ask for objects with attribute sn == Carter and which have
    // the "mail" attribute.
    String filter = "(&(sn=Carter)(mail=*))" ;
    sampleRequest.searchUsingFilterUsingSearch(filter);


    sampleRequest.getEveryAttributeUsingGetAttributes();

    sampleRequest.addANewAttribute();
    sampleRequest.removeAttributes();
    sampleRequest.replaceAttributes();
    // Generate Above all  Requests as a MemoryObject
    sampleRequest.generateDSMLRequestAsAMemoryObject();
    // until we close the context, the provider does
    // not write the search request to file.
    sampleRequest.closeContext();

  }



}

