package ejava.jndi.directory.dsml;


import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;
import java.net.URL;
import java.net.URLConnection;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
  * This example demonstrates main operations on a Directory Service
  *  Using ldap protocol handler
  *
  */

public class SearchUsingUrl
{
  Properties properties = null;

  public SearchUsingUrl(Properties properties)
  {
    super();
    this.properties = properties;
  }

  /**
   * Constructor
   * Naming Client which is initialized with a jndi.properties file.
   * Uncomment appropriate lines for each service provider in
   * jndi.properties.
   *
   * @param propertiesFileName
   *
   * @see
   */
  public SearchUsingUrl(String propertiesFileName)
    throws IOException
  {
    properties = new Properties();
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);
    // Now load the file input stream into the JNDI environment
    // properties.
    // This of course assumes that the file input stream is in a valid
    // Properties format.
    properties.load(propertiesFileStream);
  }

  public void searchEveryObjectUnderANode()
  {
      // Context From where Search Starts
      String rootContextUrl =
        (String)properties.get("java.naming.provider.url");
      // set the protocol Property
      // java.protocol.handler.pkgs=com.sun.jndi.dsmlv2.protocol
      // which is part of our properties
      String protocol =
        (String)properties.get("java.protocol.handler.pkgs");
      System.setProperty("java.protocol.handler.pkgs",protocol);


      try{
         // Create URL object with URL Provider
         URL connectingURL = new URL(rootContextUrl);
         // Open Connection to the URL
         URLConnection urlConnection = connectingURL.openConnection();
         // Read the Result into BufferedReader
         BufferedReader readResult =
            new BufferedReader(
               new InputStreamReader(urlConnection.getInputStream()));
        String currentResultLine = null;
         // Print the contents of BufferedReader on to Console.
        while((currentResultLine = readResult.readLine()) != null){
            System.out.println(currentResultLine);
        }

      }catch(Exception exception){
         exception.printStackTrace();
      }
  }

  public static void main(String[] args)
    throws Exception
  {
   String propertiesFileName = "jndi.Properties";
   SearchUsingUrl sampleSearch = null;
   try {
      // Instantiate an instance of this Naming Client
      sampleSearch =
          new SearchUsingUrl(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }
    sampleSearch.searchEveryObjectUnderANode();
  }



}

