package ejava.jndi.directory.jiniexample;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceMatches;
import net.jini.core.lookup.ServiceItem;
import net.jini.lookup.entry.Name;
import net.jini.core.entry.Entry;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.Hashtable;
import java.io.IOException;
import java.io.Serializable;
/**
  * implements the JINI Client
  */
public class CustomerJINIClient
{
  protected ServiceTemplate serviceTemplate;
  protected LookupDiscovery lookupDiscovery;
  private final int MAX_MATCHES = 5;

/**
  * it start looking for the given service
  * As soon as it finds the service, it displays the
  * found service characteristics.
  */
  public void lookForThisService(ServiceRegistrar serviceRegistrar)
  {

    ServiceMatches matches = null;

    try{
     //
      matches = serviceRegistrar.lookup(serviceTemplate,MAX_MATCHES);
    }
    catch(RemoteException remoteException){
      System.out.println(" Error during lookup :"+ remoteException);
      remoteException.printStackTrace();
      return ;
    }
    if(matches == null){
      System.out.println(" No Matching Service ");
      return  ;
    }

    System.out.println(" Found Matching Customer Service :");
    ServiceItem items[] = matches.items;
    System.out.println(" Found Matching Customer Service :"+items.length);

    for( int i = 0; i< items.length; i++){
      Customer customer = (Customer)(items[i].service);
      System.out.println("ID :" +   customer.customer_id());
      System.out.println(" First Name : " +customer.first_name());
      System.out.println(" Last Name : " +customer.last_name());
      System.out.println(" Street  : " +customer.street());
      System.out.println(" City :" +customer.city());
      System.out.println("State :" + customer.state());
      System.out.println("Zip :" +customer.zip());
      System.out.println("Email :" + customer.email());
    }

    return ;
  }

  public CustomerJINIClient()
    throws IOException
  {

    Class[] typesToLookFor = {Customer.class};

    Name n = new Name("120");
    Entry[] entries = {n};

    serviceTemplate = new ServiceTemplate(null, typesToLookFor, entries);

    //serviceTemplate = new ServiceTemplate(null, null, null);
    String[] groups = {""};
    lookupDiscovery = new LookupDiscovery(groups);


    // add the DiscoveryListener
    this.lookupDiscovery.addDiscoveryListener(
      new ClientDiscoveryListener(this));

  }

  public static void main(String args[]){
   try{
       if(System.getSecurityManager() == null){
          System.setSecurityManager(new RMISecurityManager());
       }
    CustomerJINIClient customerService =
       new CustomerJINIClient();
    // Client Discovering will take few seconds , wait till results
    // displayed
    System.in.read();

    }
    catch(IOException ioExcetpion){
      System.out.println("Error : Creating service:"+ ioExcetpion);
      ioExcetpion.printStackTrace();
      System.exit(0);
    }
    catch(Exception exception){
      System.out.println("Error : Creating service:"+ exception);
      exception.printStackTrace();
      System.exit(0);
    }
  }
}
