package ejava.jndi.directory.jiniexample;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.lookup.entry.Name;
import net.jini.core.entry.Entry;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.Hashtable;
import java.rmi.MarshalledObject;
import java.rmi.activation.Activatable;
import java.rmi.activation.ActivationDesc;
import java.rmi.activation.ActivationGroup;
import java.rmi.activation.ActivationGroupID;
import java.rmi.activation.ActivationException;
import java.rmi.activation.ActivationGroupDesc;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.StringTokenizer;
import java.util.Vector;

public class CustomerServiceRemote
{
   // max leasing time in milli seconds
  protected final int MAX_LEASING_TIME = 20*60*1000;
  protected Hashtable registrationValues = new Hashtable();
  protected LookupDiscovery discovery;
  protected ActivationGroupDesc  groupDesc;

  /**
    *
    */
  public CustomerServiceRemote()
  {
   // empty constructor;
  }

  public void registerACustomerService(Hashtable customerInformation)
    throws IOException
  {
     Integer customerID = (Integer)customerInformation.get("CUSTOMER_ID");

     Name name = new Name(""+customerID.intValue());
     System.out.println("Entry is :"+name);
     Entry[] entries = {name};
     ServiceItem serviceItem = new ServiceItem(null,
            getProxy(customerInformation), entries);
     // search the public group
     String[] groups = {""};
     discovery = new LookupDiscovery(groups);
     discovery.addDiscoveryListener(new
        CustomerDiscoveryListener(this, serviceItem));

  }

  /**
    * Constructs the proxy and returns it
  */
  public Customer getProxy(Hashtable customerInformation)
  {
    try{
     if(groupDesc == null){
       groupDesc
        = new ActivationGroupDesc(System.getProperties(),
          null);
      // register group and get id
      // Once the ActivationGroupDesc has been created, register it
      // with the activation system to obtain its ID
      //
      ActivationGroupID activationGroupID =
        ActivationGroup.getSystem().registerGroup(groupDesc);
      // Now explicitly create the group
      ActivationGroup.createGroup(activationGroupID, groupDesc, 0);
     }

      // create an Activation Descriptor
      String codeLocation =
        (String)System.getProperty("java.rmi.server.codebase");
      MarshalledObject data =
        new MarshalledObject(customerInformation);
      ActivationDesc activationDesc =
        new ActivationDesc("ejava.jndi.directory.jiniexample.CustomerActivatable",
        codeLocation,data);


      CustomerActivatableInterface customerInterface =
        (CustomerActivatableInterface)Activatable.register(activationDesc);
      return new CustomerServiceProxy(customerInterface);
    }
    catch(Exception exception){
      System.out.println("Error to get Proxy:"+exception);
      exception.printStackTrace();

      return null;
    }
  }

  public Hashtable getRegistrations()
  {
    return registrationValues;
  }

  public synchronized void registerWithLookupService(
    ServiceRegistrar serviceRegistrar, ServiceItem serviceItem)
  {
    ServiceRegistration serviceRegistration = null;
    System.out.println("registerWithLookupService");

    try{
      serviceRegistration =
        serviceRegistrar.register(serviceItem, this.MAX_LEASING_TIME);
    }
    catch(RemoteException remoteException){
      System.out.println(" Not able to Register :"+remoteException);
      remoteException.printStackTrace();
      return;
    }
    if(serviceItem.serviceID == null ){
      serviceItem.serviceID = serviceRegistration.getServiceID();
    }

    this.registrationValues.put(serviceRegistrar,serviceRegistration);
  }

   /**
    * This method reads all the customers from the file and constructs
    * them and returns, it ignores first line in the file.
    * and assumes that there is entry for every filed in each line
    */
  public static Vector getAllCustomers()
  {
    String fileName = "CustomerInfo.txt";
    Vector returnValues = null;
    try{
      FileInputStream fin = new FileInputStream(fileName);
      InputStreamReader ins = new  InputStreamReader(fin);
      BufferedReader bf =
          new BufferedReader(ins);
      String currentLine = bf.readLine();
        // ignore firstline , which has headings


      while( (currentLine = bf.readLine()) != null){
        if(returnValues == null){
          returnValues = new Vector();
        }
        Hashtable presentCustomer = new Hashtable();
        StringTokenizer st = new StringTokenizer(currentLine,",");
        while(st.hasMoreTokens()){
          String customerID = st.nextToken();
          String firstName = st.nextToken();
          String lastName = st.nextToken();
          String street = st.nextToken();
          String city = st.nextToken();
          String state = st.nextToken();
          String zip = st.nextToken();
          String email = st.nextToken();
          presentCustomer.put("CUSTOMER_ID",new Integer(customerID));
          presentCustomer.put("FIRST_NAME",firstName);
          presentCustomer.put("LAST_NAME", lastName);
          presentCustomer.put("STREET",street);
          presentCustomer.put("CITY", city);
          presentCustomer.put("STATE",state);
          presentCustomer.put("ZIP",zip);
          presentCustomer.put("EMAIL",email);
          returnValues.addElement(presentCustomer);
        }
      }
    }catch(Exception exception){
      System.out.println("Error Reading file "+exception);
      exception.printStackTrace();
    }


    return returnValues;

  }

  public static void registerCustomers()
  {
    Vector customers = getAllCustomers();
    CustomerServiceRemote customerService =
        new CustomerServiceRemote();
    for(int i = 0; i< customers.size(); i++){
      Hashtable custInfo = (Hashtable)customers.elementAt(i);
      try{
       customerService.registerACustomerService(custInfo);
      }
      catch(IOException e){
        System.out.println("Error :"+e);
        e.printStackTrace();
      }
    }

  }

  public static void main(String[] args)
  {
    try{
       if(System.getSecurityManager() == null){
          System.setSecurityManager(new RMISecurityManager());
       }


       registerCustomers();
    }
    catch(Exception excetpion){
      System.out.println("Error : Creating service:"+ excetpion);
      excetpion.printStackTrace();
      System.exit(0);
    }
  }



}
