package ejava.jndi.naming;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.CompoundName;
import javax.naming.CompositeName;
import javax.naming.Binding;
import javax.naming.Name;
import javax.naming.InvalidNameException;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * This class lists and  searches the given object in a naming or
 * directory service
 */
public class SampleListAndSearchExample extends SampleLookupExample
{

  /**
   * This constructor simply uses the super class constructor
   */
  public SampleListAndSearchExample(String propertiesFile)
   throws IOException
  {
    super(propertiesFile);
  }

  /**
   * List names in a given target
   * @param target to lookup
   */
  public void listNames(String target)
  {
    Context context;

    try {
      //get the initial context
      context = super.getInitialContext();

      NamingEnumeration namesList = context.list(target);

      if (namesList == null) {
        System.out.println(target + " contains no names ");
      } else {
        while (namesList.hasMore()) {
          //print the names in the context
          System.out.println(namesList.next());
        }
      }
    } catch (NamingException namingException) {
      System.out.println("Error : " + namingException);
    }
  }

  /**
   * list Names in a given target Name
   * @param target to lookup
   */
  public void listNames(Name target)
  {
    Context context;

    try {
      //get the initial context
      context = super.getInitialContext();

      NamingEnumeration namesList = context.list(target);

      if (namesList == null) {
        System.out.println(target + "contains no names ");
      } else {
        while (namesList.hasMore()) {
          //print the names in the context
          System.out.println(namesList.next());
        }
      }
    } catch (NamingException namingException) {
      System.out.println("Error : " + namingException);
    }
  }

  /**
   *List bindings for the given target Name
   * @param target to lookup
   */
  public void listNamesAndBindings(Name target)
  {
    Context context;

    try {
      //get the initial context
      context = super.getInitialContext();

      NamingEnumeration namesList = context.listBindings(target);

      if (namesList == null) {
        System.out.println(target + "contains no names ");
      } else {
        while (namesList.hasMore()) {
          Binding binding = (Binding) namesList.next();

          //print the names of classes and objects in the context
          System.out.println("Name is : " + binding.getName()
            + " Binding is : " + binding.getObject()+ " ");
        }
      }
    } catch (NamingException namingException) {
      System.out.println("Error :" + namingException);
    }
  }

  /**
   * list Bindings for the given target
   * @param target to lookup
   */
  public void listNamesAndBindings(String target) {
    Context context;

    try {
      // get the initial context
      context = super.getInitialContext();

      NamingEnumeration namesList = context.listBindings(target);

      if (namesList == null) {
        System.out.println(target + " contains empty names ");
      } else {
        while (namesList.hasMore()) {
          Binding binding = (Binding) namesList.next();
          System.out.println("Name is : " + binding.getName()
             + " Binding is : " + binding.getObject() + " ");
        }
      }
    } catch (NamingException namingException) {
      System.out.println("Error :" + namingException);
    }
  }

  /**
   *main method to test the SampleListAndSearchExample class
   */
  public static void main(String[] args) {

    // Ensure that a file name to lookup is passed as a parameter.
    if (args.length != 1) {
      System.out.println("Run example as: java "
      + "ejava.jndi.naming.SimpleListAndSearchExample"
      + " <TargetObjectName>");
      System.exit(0);
    }

    // Set name of properties file to use in initializing the lookup.
    String propertiesFileName = "jndi.Properties";
    SampleListAndSearchExample sampleListAndSearchExample = null;

    // Instantiate an instance of this NamingClient
    try {
      sampleListAndSearchExample =
        new SampleListAndSearchExample(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }


    // e.g. for file system Naming Provider the target bookp
    String target = args[0];

    System.out.println(
      "Listing Names using listNames(String target) method");
    sampleListAndSearchExample.listNames(target);
    System.out.println(
      "Listing Names and Bindings using listNamesAndBindings " +
        " (String target) method");
    sampleListAndSearchExample.listNamesAndBindings(target);

    try {
      String str = target;
      Properties prop
        = sampleListAndSearchExample.getContextProperties();

      // construct a Compound Name using the Compound Name
      // properties in the properties file.
      CompoundName compoundName = new CompoundName(str, prop);

      // Search the Name Repository using the CompoundName
      System.out.println("Listing Names using "
         +"listNames(Name target) method: Here target is CompoundName");
      sampleListAndSearchExample.listNames(compoundName);

      // Search the Name Repository Names and
      //     Binding Object using CompundName
      System.out.println("Listing Names and Bindings"
         + " using listNamesAndBindings(Name target) method: Here"
         + " target is CompoundName");
      sampleListAndSearchExample.listNamesAndBindings(compoundName);

      // CompositeName is heirarchial name space, the atomic parts
      // of the CompositeName may be CompoundName.
      String strName1 = compoundName.toString();
      CompositeName compositeName = new CompositeName(strName1);

      // Search the Name Repository using the CompoundName
      System.out.println("Listing Names using listNames(Name target)"
         + " method: Here target is CompositeName");
      sampleListAndSearchExample.listNames(compositeName);
      System.out.println("Listing Names and Bindings using"
         + " listNamesAndBindings(Name target) method: Here"
         + " target is CompoundName");

      // Search the Name Repository Names and Binding Object using
      //CompundName
      sampleListAndSearchExample.listNamesAndBindings(compositeName);
    } catch (InvalidNameException invalidNameException) {
      System.out.println("Error :" + invalidNameException);
      System.exit(0);
    }
  }
}


