package ejava.jndi.naming;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Hashtable;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * This example looks up an object for a given name.
 */
public class SampleLookupExample
{
  private Context context;
  private Properties contextProperties = new Properties();

  /**
   *Constructor
   *
   *
   * @see
   */
  public SampleLookupExample()
  {
    super();
  }
  /**
   *Constructor
   * Naming Client which is initialized with a jndi.properties file.
   *
   * @param propertiesFileName
   *
   * @see
   */
  public SampleLookupExample(String propertiesFileName)
    throws IOException
  {
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);

    // Now load the file input stream into the JNDI environment
    // properties.This of course assumes that the file input stream is
    // in a valid Properties format.
    contextProperties.load(propertiesFileStream);
  }




  /**
   * Constructor
   *
   *
   * @param systemProperties
   *
   * @see
   */
  public SampleLookupExample(Properties contextProperties)
  {
    this.contextProperties = contextProperties;
  }

  /**
   * Method to create initial context
   *
   *
   * @return initial context
   *
   * @throws NamingException
   *
   * @see
   */
  public Context getInitialContext()
    throws NamingException
  {

    // Create initial context if it is not already created
    if (context == null) {
      // Read in a set of default paramters.
      context = new InitialContext(contextProperties);
    }
    return context;
  }

  /**
   * Method
   *
   * Look up the  object associated with the name
   * @param name to look up
   *
   * @return object matched with name
   *
   * @see
   */
  public java.lang.Object lookupBoundObject(String name)
    throws NamingException
  {
    java.lang.Object boundObject = null;

    // Retrieve initial context from our NamingInterface class
    if(context == null){
      getInitialContext();
    }


    // Now lookup the bound object with this name
    try {
      boundObject = context.lookup(name);
    } catch (NamingException namingException) {
      System.out.println("Error : " + namingException);
      namingException.printStackTrace();
      return null;
    }

    System.out.println(" Name is : " + name +
      "    Bound Object is : "+ boundObject);

    return boundObject;
  }

  /**
   * Returns the context properties
   */
  public Properties getContextProperties()
  {
    return contextProperties;
  }

  /**
   * Main Test Driver:
   *       java ejava.jndi.naming.SampleLookupExample <ObjectNameToLookup>
   * @param args command line argument
   *
   * @see
   */
  public static void main(String[] args)
    throws Exception
  {
    if (args.length != 1) {
      System.out .println("Run example as: java "
         + "ejava.jndi.naming.SampleLookupExample <ObjectNameToLookup>");
      System.exit(0);
    }
    // Set name of properties file to use in initializing the simple
    // lookup.
    String propertiesFileName = "jndi.Properties";
    // Set name of properties file to use in initializing the
    // simple lookup.
    // Instantiate an instance of this NamingClient
    SampleLookupExample sampleLookupExample = null;
    sampleLookupExample = new SampleLookupExample(propertiesFileName);
    // Demonstrate lookup of a bound object name
    String name = args[0];
    System.out.println("Going to Lookup " + name);
    sampleLookupExample.lookupBoundObject(name);
  }

}


