
package ejava.jndi.naming;

import javax.rmi.PortableRemoteObject;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Properties;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.io.FileInputStream;
import java.io.IOException;
import java.rmi.RMISecurityManager;
import javax.naming.Context;
import javax.naming.NameNotFoundException;

/**
 * RMIIIOP Server class implementation
 */
public class SampleRMIIIOPServer extends PortableRemoteObject
  implements Sample
{

  /**
   * Constructor
   */
  public SampleRMIIIOPServer()
      throws RemoteException
  {
    super();
  }

  /**
   * Implementation of Sample Interface only method
   * @param s input string
   * @return string to return after manipulation
   */
  public String sayHi(String s)
    throws RemoteException
  {
    return "You Sent:" + s;
  }

  /**
   * This method binds the object with different names
   * @param context to get names to bind
   */
  public void bindWithDifferentNames(Context context)
          throws NamingException
  {
    Properties prop = (Properties) context.getEnvironment();
    String namingServerURL =
      (String) prop.get("java.naming.provider.url");
    String simpleName1 = (String) prop.get("SIMPLE_NAME1");
    String simpleName2 = (String) prop.get("SIMPLE_NAME2");

    // binding this object with different Names
    context.bind(namingServerURL + simpleName1, this);
    System.out.println("Bound As :" + namingServerURL + simpleName1);
    context.bind(namingServerURL + simpleName2, this);
    System.out.println("Bound As :" + namingServerURL + simpleName2);
  }

  /**
   * main method to test the functionality
   */
  public static void main(String[] args) {
    if (args.length == 0) {
      System.out.println("Usage : java -Djava.security.policy"
       + "=rmi.policy ejava.jndi.naming.SampleRMIIIOPServer "
       + "jndi.Properties");
      System.exit(0);
    }

    String propertiesFileName = args[0];

    try {
      FileInputStream fin = new FileInputStream(propertiesFileName);
      Properties properties = new Properties();

      properties.load(fin);

      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new RMISecurityManager());
      }

      Context namingContext =
        new InitialContext((Hashtable) properties);
      SampleRMIIIOPServer serverObject = new SampleRMIIIOPServer();

      serverObject.bindWithDifferentNames(namingContext);
      System.out.println("Bound : Waiting for clients...");
    }
    catch (IOException ioException) {
      System.out.println("Failed to open properties :" + ioException);
      System.exit(0);
    }
    catch (NamingException namingException) {
      System.out.println("Failed to bind :" + namingException);
      System.exit(0);
    }
  }
}


