package ejava.jsp.tagext;

import ejava.servlets.Item;
import ejava.servlets.ShoppingCart;
import ejava.servlets.TShirt;

import java.io.IOException;
import java.util.Vector;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.IterationTag;

public class CartItemsTag extends BodyTagSupport
{
    private int count;
    private ShoppingCart cart;
    private Vector items;

    public CartItemsTag()
    {
    }

    /**
     *  Called by the container when the start tag is processed, but
     *  before any body contents have been read
     */
    public int doStartTag() throws JspException
    {
        // returning EVAL_BODY_BUFFERED will cause 
        // doInitBody() to be invoked before the first
        // pass through the body of the tag 
        return BodyTagSupport.EVAL_BODY_BUFFERED;
    }

    /**
     *  Called when the closing tag is read
     */
    public int doEndTag() throws JspException
    {
        return BodyTagSupport.EVAL_PAGE;
    }


    /**
     *  Initializes the values that will be dynamically
     *  inserted into the body of the tag
     */
    public void doInitBody() throws JspException
    {
	count = 0;
	cart = (ShoppingCart)pageContext.findAttribute
	    (ShoppingCart.SHOPPING_CART_OBJECT);
	items = cart.getItems();

        // initialize scripting variables for display on page
        setScriptingVariables();
    }

    /**
     *  Called after each successive processing of the body contents
     */
    public int doAfterBody() throws JspException
    {

        // then increment counter
        count++;

        // and decide to continue iterating or not
        if (count < items.size())
        {
            setScriptingVariables();
            return BodyTagSupport.EVAL_BODY_AGAIN;
        }
        else
        {
	    // after all iterations are complete, send the whole thing
	    try
            {
                System.out.println(bodyContent.getString());
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
            catch (IOException e)
            {
            }
            // and continue on with the rest of the page
            return BodyTagSupport.SKIP_BODY;
        }
    }

    private void setScriptingVariables()
    {
        if (count >= items.size())
            return;

        TShirt shirt = (TShirt)items.get(count);

        pageContext.setAttribute("item_id", 
                                 new Integer(shirt.getItemID()));
        pageContext.setAttribute("item_quantity", 
                                 new Integer(shirt.getQuantity()));
        pageContext.setAttribute("item_price", 
                                 new Float(shirt.getTotalPrice()));
        pageContext.setAttribute("item_unit_price", 
                                 new Float(shirt.getUnitPrice()));
        pageContext.setAttribute("item_size", shirt.getSize()); 
        pageContext.setAttribute("item_color", shirt.getColor());
        pageContext.setAttribute("item_design_front", 
                                 shirt.getDesignFront());
        pageContext.setAttribute("item_design_back", 
                                 shirt.getDesignBack());
        pageContext.setAttribute("item_front_image", 
                                 shirt.getPictureFrontFileName());
        pageContext.setAttribute("item_back_image", 
                                 shirt.getPictureBackFileName());
         
    }

    /**
     *  Called when this tag exits to free any resources
     */
    public void release()
    {
        cart = null;
        items = null;
    }
}
