package ejava.jsp.tagext;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;
import javax.servlet.http.HttpServletRequest;

/**
 *  Tag designed to emulate a standard HTML <a href=..> tag
 *  but takes an additional attribute "secure", which will
 *  rebuild the URL starting with "https://hostname" instead
 *  of the normal relative URL
 */
public class SecureHrefTag extends SimpleTagSupport
{
    private String href;
    private Boolean secure;

    public SecureHrefTag()
    {
        secure = new Boolean(false);
    }

    public void doTag() throws JspException, IOException
    {
        HttpServletRequest request = 
          (HttpServletRequest)jspContext.findAttribute(
                                            PageContext.REQUEST);
        String newHref;
        if (secure.booleanValue())
        {
	    // change http to https
            newHref = "https://" + request.getServerName() + href;
	}
        else
	    newHref = href;
  
        JspWriter out = getJspContext().getOut();
        // start the <a href tag
        out.write("<a href='" + newHref + "'>");
        // write out the body
        getJspBody().invoke(out);
        // close the tag
        out.write("</a>");
    }

    public void setHref(String href)
    {
        this.href = href;
    }

    public void setSecure(Boolean secure)
    {
        this.secure = secure;
    }
}


