package ejava.rmi.queryserviceiiop;

import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.util.Vector;
import javax.naming.InitialContext;
import javax.naming.Context;
import java.util.prefs.Preferences;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NameComponent;
import org.omg.CORBA.ORB;
import ejava.rmi.ClientReceiver;
import ejava.rmi.Customer;
import ejava.rmi.QueryServer;
import ejava.rmi.Order;

public class QueryClient
{

  public static void main(String[] args)
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);

    try {
      // Set security manager
      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new RMISecurityManager());
      }      
      
      // Get orb reference
      System.out.println("Getting initial Context");
      ORB orb = ORB.init(args, System.getProperties());

      // Get the root naming context
      org.omg.CORBA.Object objRef =
        orb.resolve_initial_references("NameService");
      NamingContext context = NamingContextHelper.narrow(objRef);

      String name = preferences.get("QUERY_SERVER_NAME",null);
      // Resolve the object reference in naming
      NameComponent nc = new NameComponent(name, "");
      NameComponent path[] = {nc};
      System.out.println("Client Looking for Server :" + name);
      System.out.println("Type Enter Key");
      System.in.read();    // to hold the screen

      // Lookup OrderManager reference
      org.omg.CORBA.Object object = context.resolve(path);
      System.out.println(object);
      QueryServer qserver =
        (QueryServer)PortableRemoteObject.narrow(object,
                               ejava.rmi.QueryServer.class);
      System.out.println("Client querying for customers :");
      System.out.println("Type Enter Key");
      System.in.read();    // to hold the screen

      // Query for customers and traverse results
      Vector v = qserver.findCustomersWhoOrderedForMoreThan(10.0f);

      System.out.println("Result Customers :" + v);

      for (int i = 0; i < v.size(); i++) {
        String cid = (String) v.elementAt(i);

        nc = new NameComponent(cid, "");
        NameComponent path1[] = {nc};

        object = context.resolve(path1);

        Customer customer = (Customer)
          PortableRemoteObject.narrow(object, Customer.class);

        System.out.println(i + " : is " + customer);
        System.out.println("FirstName :" + customer.getFirstName());
        System.out.println("LastName:" + customer.getLastName());
      }

      System.out.println("Server Writing Orders ");
      System.out.println("Type Enter Key");
      System.in.read();    // to hold the screen

      // Register client callback with query server
      ClientReceiverImplementation cr =
        new ClientReceiverImplementation();
      NameComponent nc1 = new NameComponent("ClientReceiver", "");
      NameComponent path2[] = {nc1};
      java.rmi.Remote stub = PortableRemoteObject.toStub(cr);

      context.rebind(path2, (org.omg.CORBA.Object) stub);
      qserver.register("ClientReceiver", cr);

      // Induce callbacks to be made by requesting order info
      qserver.findOrdersWhichAreFrom("VA");

    } catch (Exception exception) {
      System.out.println("Error: " + exception);
      exception.printStackTrace();
      System.exit(0);
    }
  }

}

