package ejava.rmi.queryserviceiiop;

import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NameComponent;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;
import java.rmi.RMISecurityManager;
import java.util.Vector;
import java.util.prefs.Preferences;
import java.sql.Connection;
import java.sql.SQLException;
import java.lang.ClassCastException;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.DriverManager;
import java.sql.Date;
import ejava.rmi.ClientReceiver;
import ejava.rmi.Customer;
import ejava.rmi.QueryServer;
import ejava.rmi.Order;

public class QueryServerImplementation extends PortableRemoteObject
  implements QueryServer
{
  private ClientReceiver clientReceiver;
  private String clientReveiverName;
  private Connection connection;
  private Properties properties;
  public NamingContext context;

  public QueryServerImplementation()
    throws RemoteException
  {
    super();
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);
    // Get connection properties from preferences
    String driverClass = preferences.get("DRIVER_CLASS", null);
    String dbURL   = preferences.get("DATABASE_URL",null);
    String userName = preferences.get("USER_NAME",null);
    String password = preferences.get("PASSWORD",null);
    try {
      Class.forName(driverClass);

      if (userName == null || password == null || userName.length() == 0
            || password.length() == 0) {
        connection = DriverManager.getConnection(dbURL);
      } else {
        connection =
          DriverManager.getConnection(dbURL, userName,password);
      }
    } catch (ClassNotFoundException cnfEx) {
      System.out.println("Driver Class Not found " + cnfEx);
    } catch (SQLException sqlEx) {
      System.out.println("SQL Exception :" + sqlEx);
    }
  }

  public ResultSet getResultSet(String sqlQuery,
                                java.lang.Object whereValue)
    throws SQLException
  {
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlQuery);
    preparedStatement.setObject(1, whereValue);
    ResultSet rs = preparedStatement.executeQuery();
    return rs;
  }

  public Vector findCustomersWhoOrderedForMoreThan(float value)
    throws RemoteException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);

    // Get customer query
    String customerQueryString =
      preferences.get("CUSTOMER_QUERY",null);
    if (customerQueryString == null) {
      throw new RemoteException("CUSTOMER_QUERY in  file is null");
    }
    // Issue query and create/bind Customer server for each result
    //  ...also return Vector of customer IDs
    try {
      ResultSet rs =
        this.getResultSet(customerQueryString,new Float(value));
      ResultSetMetaData rsMetaData = rs.getMetaData();
      int nColumns = rsMetaData.getColumnCount();
      Vector customerIDs = new Vector();

      while (rs.next()) {
        String customerID = rs.getString("CUSTOMER_ID");
        String firstName = rs.getString("FIRST_NAME");
        String lastName = rs.getString("LAST_NAME");
        String address1 = rs.getString("ADDRESS_1");
        String address2 = rs.getString("ADDRESS_2");
        String city = rs.getString("CITY");
        String state = rs.getString("STATE");
        String phoneNumber = rs.getString("PHONE");
        String eMail = rs.getString("EMAIL");
        String zip = rs.getString("ZIP_CODE");
        CustomerImplementation custImpl =
          new CustomerImplementation();
        custImpl.setFirstName(firstName);
        custImpl.setLastName(lastName);
        custImpl.setAddress1(address1);
        custImpl.setAddress2(address2);
        custImpl.setCity(city);
        custImpl.setState(state);
        custImpl.setZip(zip);
        custImpl.setEmailAddress(eMail);
        custImpl.setPhoneNumber(phoneNumber);
        String name = customerID;
        custImpl.setContext(this.context);
        custImpl.bind(name);
        customerIDs.addElement(customerID);
      }

      return customerIDs;
    } catch (SQLException sqlEx) {
      throw new RemoteException("failed to getCustomers", sqlEx);
    }
  }

  /**
   * Method
   *
   * this method returns the customer remote reference
   * binding names.
   */
  public void findOrdersWhichAreFrom(String state)
    throws RemoteException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);
    String orderQueryString = preferences.get("ORDER_QUERY",null);

    // Get orders query
    if (orderQueryString == null) {
      throw new RemoteException("ORDER_QUERY in  file is null");
    }
    // Issue orders query and callback client with each Order object
    try {
      ResultSet rs = this.getResultSet(orderQueryString, state);
      ResultSetMetaData rsMetaData = rs.getMetaData();
      int nColumns = rsMetaData.getColumnCount();
      Vector orders = new Vector();

      while (rs.next()) {
        int orderID = rs.getInt("ORDER_ID");
        Date orderDate = rs.getDate("ORDER_DATE");
        double orderValue = rs.getDouble("TOTAL_PRICE");
        Order order = new Order(orderID, orderDate, orderValue);
        orders.addElement(order);
      }

      if (this.clientReceiver != null) {
        clientReceiver.setResultOrders(orders);
      }
    } catch (SQLException sqlEx) {
      throw new RemoteException("failed to getCustomers", sqlEx);
    }
  }


  /**
   * register ClientReceiver instance to the server.
   */
  public void register(String name, ClientReceiver clientReceiver)
    throws RemoteException
  {
    System.out.println("Registering the ClientReceiver :"
                         + clientReceiver);

    this.clientReceiver = clientReceiver;
    this.clientReveiverName = name;
  }

  public static void main(String[] args)
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);

    try {
      // Get query.server.name
      String name = preferences.get("QUERY_SERVER_NAME",null);
      
      // Instantiate Query Server Impl
      QueryServerImplementation queryServerImpl 
        = new QueryServerImplementation();
        
      // Set security manager
      System.setSecurityManager(new RMISecurityManager());

      // Get Name Service reference and bind server to name service
      org.omg.CORBA.ORB orb 
        = org.omg.CORBA.ORB.init(args, System.getProperties());

      System.out.println("Getting Naming Reference");
      org.omg.CORBA.Object namingReference
        = orb.resolve_initial_references("NameService");

      System.out.println("Getting Context Reference and stub");
      queryServerImpl.context
        = NamingContextHelper.narrow(namingReference);
      java.rmi.Remote stub 
        = PortableRemoteObject.toStub(queryServerImpl);

      NameComponent nc = new NameComponent(name, "");
      NameComponent path[] = {nc};

      System.out.println("Name Binding :"+ path);
      queryServerImpl.context.rebind(path,(org.omg.CORBA.Object) stub);

      System.out.println("Server Waiting");
    } catch (Exception ne) {
      System.out.println("Error   :" + ne);
    }
  }

}

