package ejava.rmi.queryservicejrmp;

import java.lang.ClassCastException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.Remote;
import java.rmi.Naming;
import java.net.MalformedURLException;
import java.rmi.RMISecurityManager;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.DriverManager;
import java.sql.Date;
import java.util.Vector;
import java.util.prefs.Preferences;
import ejava.rmi.ClientReceiver;
import ejava.rmi.Customer;
import ejava.rmi.QueryServer;
import ejava.rmi.Order;

public class QueryServerImplementation extends UnicastRemoteObject
  implements QueryServer
{
  ClientReceiver clientReceiver;
  String clientReceiverName;
  Connection connection;

  public QueryServerImplementation()
    throws RemoteException
  {
    super();
    Preferences preferences =
      Preferences.userNodeForPackage(
         ejava.rmi.QueryServer.class);
    // Get connection properties from preferences
    String driverClass = preferences.get("DRIVER_CLASS", null);
    String dbURL   = preferences.get("DATABASE_URL",null);
    String userName = preferences.get("USER_NAME",null);
    String password = preferences.get("PASSWORD",null);
    // Get Connection based on prefrences information
    try {
      Class.forName(driverClass);
      if (userName == null || password == null ||
            userName.length() == 0|| password.length() == 0) {
        connection = DriverManager.getConnection(dbURL);
      } else {
        connection =
          DriverManager.getConnection(dbURL, userName,password);
      }
    } catch (ClassNotFoundException cnfEx) {
      System.out.println("Driver Class Not found " + cnfEx);
    } catch (SQLException sqlEx) {
      System.out.println("SQL Exception :" + sqlEx);
    }
  }

  public ResultSet getResultSet(String sqlQuery,  Object whereValue)
    throws SQLException
  {
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlQuery);

    preparedStatement.setObject(1, whereValue);

    ResultSet rs = preparedStatement.executeQuery();

    return rs;
  }

  public Vector findCustomersWhoOrderedForMoreThan(float value)
    throws RemoteException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);

    // Get customer query
    String customerQueryString =
      preferences.get("CUSTOMER_QUERY",null);
    if (customerQueryString == null) {
      throw new RemoteException("CUSTOMER_QUERY in  file is null");
    }

    // Issue query and create/bind Customer server for each result
    //  ...also return Vector of customer IDs
    try {
      ResultSet rs = this.getResultSet(customerQueryString,
                                       new Float(value));
      ResultSetMetaData rsMetaData = rs.getMetaData();
      int nColumns = rsMetaData.getColumnCount();
      Vector customerIDs = new Vector();

      while (rs.next()) {
        String customerID = rs.getString("CUSTOMER_ID");
        String firstName = rs.getString("FIRST_NAME");
        String lastName = rs.getString("LAST_NAME");
        String address1 = rs.getString("ADDRESS_1");
        String address2 = rs.getString("ADDRESS_2");
        String city = rs.getString("CITY");
        String state = rs.getString("STATE");
        String phoneNumber = rs.getString("PHONE");
        String eMail = rs.getString("EMAIL");
        String zip = rs.getString("ZIP_CODE");
        CustomerImplementation custImpl = new CustomerImplementation();

        custImpl.setFirstName(firstName);
        custImpl.setLastName(lastName);
        custImpl.setAddress1(address1);
        custImpl.setAddress2(address2);
        custImpl.setCity(city);
        custImpl.setState(state);
        custImpl.setZip(zip);
        custImpl.setEmailAddress(eMail);
        custImpl.setPhoneNumber(phoneNumber);
        String registryURL =
          preferences.get("REGISTRY_SERVER_URL",null);
        String name = registryURL + customerID;
        System.out.println(name);
        custImpl.bind(name);
        customerIDs.addElement(customerID);
      }
      return customerIDs;
    } catch (SQLException sqlEx) {
      throw new RemoteException("failed to getCustomers", sqlEx);
    }
  }

  public void findOrdersWhichAreFrom(String state)
    throws RemoteException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(
                                     ejava.rmi.QueryServer.class);
    String orderQueryString = preferences.get("ORDER_QUERY",null);

    // Get orders query
    if (orderQueryString == null) {
      throw new RemoteException("ORDER_QUERY in  file is null");
    }
    // Issue orders query and callback client with each Order object
    try {
      ResultSet rs = this.getResultSet(orderQueryString, state);
      ResultSetMetaData rsMetaData = rs.getMetaData();
      int nColumns = rsMetaData.getColumnCount();
      Vector orders = new Vector();

      while (rs.next()) {
        int orderID = rs.getInt("ORDER_ID");
        Date orderDate = rs.getDate("ORDER_DATE");
        double orderValue = rs.getDouble("TOTAL_PRICE");
        Order order = new Order(orderID, orderDate,
                                orderValue);

        orders.addElement(order);
      }

      if (this.clientReceiver != null) {
        clientReceiver.setResultOrders(orders);
      }
    } catch (SQLException sqlEx) {
      throw new RemoteException("failed to getCustomers", sqlEx);
    }
  }


  /**
   * register ClientReceiver instance with the server.
   */
  public void register(String name, ClientReceiver clientReceiver)
    throws RemoteException
  {
    this.clientReceiver = clientReceiver;
    this.clientReceiverName = name;
  }

  public static void main(String[] args)
  {
    Preferences preferences =
      Preferences.userNodeForPackage(ejava.rmi.QueryServer.class);
    try{
      // Read Preferences from Preferences
      String name = preferences.get("QUERY_SERVER_NAME",null);
      QueryServerImplementation queryServerImpl =
        new QueryServerImplementation();
      // Set security manager
      System.setSecurityManager(new RMISecurityManager());
      // Get registry URL and bind query server to registry
      String registryURL =
        preferences.get("REGISTRY_SERVER_URL",null);
      System.out.println("Binding :" + registryURL + name);
      Naming.rebind(registryURL + name, queryServerImpl);
      System.out.println("Bound:");
    }catch(RemoteException re){
      System.out.println("Remote Exception :"+re);
      re.printStackTrace();
    }catch (MalformedURLException malformedURLException) {
      System.out.println("RemoteException: " + malformedURLException);
      malformedURLException.printStackTrace();
    }
  }

}

