package ejava.servlets;

// 
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.Enumeration;

// 
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
  * This servlet is used to handle order status information requests
  */
public class OrderStatusServlet  extends HttpServlet
{
   public final static  String SERVLET_NAME = "/OrderStatusServlet";
   private final String LOGIN_BUTTON_VALUE = "LOGIN";
   private final String PASSWORD_REMINDER_BUTTON_VALUE="REMIND";
   private final String NUMBER_OF_TIMES_FAILED = "FAILED_TO_LOGIN_COUNT";
   private final int MAX_NUMBER_OF_LOGIN_ALLOWED = 5;
   private final String PASSWORD_FIELD_NAME = "PASSWORD";
   private final String EMAIL_FIELD_NAME = "EMAIL";
   private final String EMAIL_LOOKUP_FIELD_NAME = "EMAIL_LOOKUP";


/**
  * Called when a GET request is made to display login form
  */
  public void doGet(HttpServletRequest servletRequest, HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write the body of the page for the order status 
    String message = "Enter Email And Password to retrieve your Orders.";
    writePageBodyForLoginForm(printWriter, servletRequest,message);

    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * Write the complete page for the login form
  */
  private void writePageBodyForLoginForm(PrintWriter printWriter,
    HttpServletRequest servletRequest, String message)
  {
    // Write the op half and left hand side of page
    ServletsHelper.writePageTopArea(printWriter);
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"100%\" >");
    printWriter.println(" <TR> ");
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write the center part of page as a login form
    writePageCenterAreaForLoginForm(printWriter, servletRequest, message);
    // close the TABLE and document
    printWriter.println("</TR>");
    printWriter.println("</TABLE> <br>&nbsp;");
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * Write the center part of a page as a login form
  */
  private void   writePageCenterAreaForLoginForm(PrintWriter printWriter,
    HttpServletRequest servletRequest, String message)
  {
    printWriter.println("<TD VALIGN=TOP>");
    printWriter.println("<STRONG>"+message+
      "   <STRONG>");
    printWriter.println("<FORM METHOD=\"POST\" NAME=\"loginForm\" "+
      "ACTION="+servletRequest.getContextPath()+SERVLET_NAME+"> ");
    printWriter.println("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0"+
      " WIDTH=\"220\" BGCOLOR=\"#FFCC66\" > ");
    printWriter.println("<TR> <TD COLSPAN=\"2\" >" +
      " <INPUT SIZE=\"26\" NAME=" + EMAIL_FIELD_NAME +"  value=\"\"> "+
      " <FONT face=\"arial, sans-serif\" > " +
      "<FONT SIZE=-2>E-MAIL ADDRESS: "+
      "<FONT color=\"#990033\">  *</FONT></FONT></FONT> </TD> </TR> ");
    printWriter.println("<TR> <TD COLSPAN=\"2\" > "+
      " <INPUT TYPE = \"PASSWORD\" SIZE=\"26\"  NAME=" + PASSWORD_FIELD_NAME+
      " value=\"\"> " +
      " <FONT face=\"arial, sans-serif\"> <FONT SIZE=-2>PASSWORD " +
      " <FONT color=\"#990033\">  *</FONT></FONT></FONT> </TD> </TR> ");
    printWriter.println("<TR> <TD> "+
      " <INPUT type=SUBMIT Name = \"Login\" Value="+LOGIN_BUTTON_VALUE+
      "> </TD> </TR> ");
    printWriter.println(" <TR> <TD COLSPAN=\"2\" >" +
      " <FONT face=\"arial, sans-serif\">"+
      " <FONT SIZE=-2>For Password Reminder Enter Email"+
      "<FONT color=\"#990033\">  *</FONT></FONT></FONT> </TD> </TR> ");
    printWriter.println(" <TR> <TD COLSPAN=\"2\" > " +
      " <INPUT SIZE=\"26\" NAME="+EMAIL_LOOKUP_FIELD_NAME+" value= > "+
      " <FONT face=\"arial, sans-serif\"> <FONT SIZE=-2> EMAIL ADDRESS:"+
      "  <FONT color=\"#990033\">  *</FONT></FONT></FONT> </TD> </TR> ");
    printWriter.println("<TR> <TD>"+
      " <INPUT type=SUBMIT Name = \"Lookup\" Value=" +PASSWORD_REMINDER_BUTTON_VALUE+
      "> </TD> </TD> </TR> ");
    printWriter.println("</TABLE> </FORM> </TD>");

  }

/**
  * Determine if user pressed a remind request button and send dummy email
  */
  private boolean handleRemindRequest(String[] remindValues,
       HttpServletRequest servletRequest,
       HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    if(remindValues != null && remindValues.length != 0 &&
      remindValues[0].trim().length() != 0){
      String emailAddress =  remindValues[0];
      writeEmailReminderPage(emailAddress,
                              servletRequest, servletResponse);
      return true;
    }
    else{
      return false;
    }
  }

/**
  * Handle the POST request sent to this servlet
  */
  public void doPost(HttpServletRequest servletRequest, HttpServletResponse servletResponse)
    throws ServletException, IOException
 {
    // Check whether the user is asking to remind him/her
    // This occurs when remind button is pressed
    String[] remindValues  =
      servletRequest.getParameterValues(EMAIL_LOOKUP_FIELD_NAME);

    if(handleRemindRequest(remindValues,
                           servletRequest, servletResponse)){
      return;
    }

    // Otherwise user hit Login button
    // Get email and password info
    String emailAddress =
      servletRequest.getParameterValues(EMAIL_FIELD_NAME)[0];
    String password =
      servletRequest.getParameterValues(PASSWORD_FIELD_NAME)[0];
    log("User Entered EMAIL :"+emailAddress);
    log("PASSWORD :" +password);

    // Retrieve session info
    boolean newSession = false;
    HttpSession session = servletRequest.getSession(newSession);
    if(session == null){
      newSession = true;
      session = servletRequest.getSession(newSession);
    }

    // If cannot login, then implement login time checking logic
    if(!login(session, emailAddress, password)){
      Integer nTimesTried = (Integer)session.getAttribute(NUMBER_OF_TIMES_FAILED);
      if(nTimesTried == null ){
        session.setAttribute(this.NUMBER_OF_TIMES_FAILED, new Integer(1));
        informUserThatLoginFailed(servletRequest, servletResponse);
      }else if(nTimesTried.intValue() < MAX_NUMBER_OF_LOGIN_ALLOWED){
        informUserThatLoginFailed(servletRequest, servletResponse);
        session.setAttribute(this.NUMBER_OF_TIMES_FAILED,
          new Integer(nTimesTried.intValue()+1));

      }else{
        // If the user tries more than MAX_NUMBER_OF_LOGIN_ALLOWED,
        // then ask the user to register again.
        askTheUserToRegister(servletRequest, servletResponse);
      }
    }
    else{ // else if could login, show dummy order status
      showUserOrders(servletRequest, servletResponse);
    }
  }

/**
  * Write information for failed login
  */
  private void informUserThatLoginFailed(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException , IOException
  {
    log(" Failed to Login :" );

    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write the page body login form
    String message = "Failed to Login! Please Enter Correct Email Address and Password.";
    writePageBodyForLoginForm(printWriter, servletRequest,message);
    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * This method handles the login for a user
  */
  private boolean login(HttpSession session, String emailAddress,
    String password) throws ServletException
  {
    // Get init parameter names
    Enumeration names = this.getInitParameterNames();
    // Log name and value info from initial parameters
    while(names.hasMoreElements()){
      String name = (String)names.nextElement();
      String value = (String)getInitParameter(name);
      log("Name :"+name+"Value :"+value);
    }
    log("entered Login :");

    // Get email and password from init parameters
    String emailAddressFromParameter = getInitParameter(Customer.EMAIL);
    String passwordFromParameter = getInitParameter(Customer.PASSWORD);
    log("User Entered EMAIL :"+emailAddressFromParameter);
    log("PASSWORD :" +passwordFromParameter);
    // If email/password from init parameters match user entered values,
    // then construct some default user information
    if(emailAddressFromParameter.equalsIgnoreCase(emailAddress)
      && passwordFromParameter.equalsIgnoreCase(password) ){
      constructDefaultUserInformation(session);
      return true;
    }
    return false;
  }

/**
  * Once failed to login a certain number of times,
  * ask the user to register again.
  */
  private void askTheUserToRegister(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException , IOException
  {
    String contextName = servletRequest.getContextPath();
    String url = "HTTP://"+ServletsHelper.getServerInfo(servletRequest)+
          RegistrationServlet.REGISTRATION_HTML;
    servletResponse.sendRedirect(url);
  }

/**
  * Dummy method to look up the user information for the
  * given email address and send the password to the user.
  */
  private void writeEmailReminderPage(String emailAddress,
    HttpServletRequest servletRequest, HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    log(" Send Eamil:" );

    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Create top half of page and left hand side (buttons)
    ServletsHelper.writePageTopArea(printWriter);
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"300%\" >");
    printWriter.println(" <TR> ");
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write out statement that email is being sent to user 
    String message = "Send Password in Email to :"+emailAddress;
    printWriter.println("<TD VALIGN=TOP>" );
    printWriter.println("<STRONG>"+message+" <STRONG>" );
    printWriter.println(" </TD> ");
    printWriter.println("</TR>");
    printWriter.println("</TABLE>");

    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * Read the dummy user information from init parameters
  * and construct the Customer object and store in session
  */
  private void constructDefaultUserInformation(
    HttpSession session) throws ServletException
  {
    String firstName =
      getInitParameter(Customer.FIRST_NAME);
    String lastName =
      getInitParameter(Customer.LAST_NAME);
    String middleName =
      getInitParameter(Customer.MIDDLE_NAME);
    String password =
      getInitParameter(Customer.PASSWORD);
    String phone =
      getInitParameter(Customer.PHONE);
    String eMail =
      getInitParameter(Customer.EMAIL);
    String address_1 =
      getInitParameter(Address.ADDRESS_1);
    String address_2 =
      getInitParameter(Address.ADDRESS_2);
    String city =
      getInitParameter(Address.CITY);
    String state =
      getInitParameter(Address.STATE);
    String zip_code =
      getInitParameter(Address.ZIP_CODE);

    Address address = new Address(address_1, address_2, city,
      state,zip_code);
    Customer customer =
      new Customer(firstName, lastName,
      middleName,address,phone,eMail,password);

    // Keep the customer information in the session for future use.
    session.setAttribute(Customer.CUSTOMER_OBJECT , customer);

  }
/**
  * Read the dummy order information from init parameters
  * and construct the Order object and store in session
  */
  private void constructOrderForThisCustomer(HttpServletRequest servletRequest)
    throws ServletException
  {
    // Get init parameters
    Enumeration parameters = this.getInitParameterNames();
    while(parameters.hasMoreElements()){
      String name = (String)parameters.nextElement();
      String value = this.getInitParameter(name);
      log("Name :"+name +"Value :"+value);
    }
    // Get order information
    String orderID =
      getInitParameter(Order.ORDER_ID);
    String orderDate =
      getInitParameter(Order.ORDER_DATE);
    String shipInstruction =
      getInitParameter(Order.SHIP_INSTRUCTION);
    String shipDate =
      getInitParameter(Order.SHIP_DATE);
    String shipWeight =
      getInitParameter(Order.SHIP_WEIGHT);
    String paidDate =
      getInitParameter(Order.PAID_DATE);
    // Get item information
    String itemID =
      getInitParameter(Item.ITEM_ID);
    String quantity =
      getInitParameter(Item.QUANTITY);
    String totalPrice =
      getInitParameter(Item.TOTAL_PRICE);
   // Get TShirt information
   String shirtID =
      getInitParameter(TShirt.SHIRT_ID);
   String size =
      getInitParameter(TShirt.SIZE);
   String color =
      getInitParameter(TShirt.COLOR);
   double priceForItem =
    Double.parseDouble((String)getInitParameter(TShirt.PRICE_FOR_ONE_ITEM));
   String designFront =
      getInitParameter(TShirt.DESIGN_FRONT);
   String designBack =
      getInitParameter(TShirt.DESIGN_BACK);
   String pictureFront =
      getInitParameter(TShirt.PICTURE_FRONT);
   String pictureBack =
      getInitParameter(TShirt.PICTURE_BACK);
   // Attempt to get Customer information from session
   HttpSession session = servletRequest.getSession();
   Customer customer =  (Customer)session.getAttribute(Customer.CUSTOMER_OBJECT);
   // Get Address information from Customer
   Address billingAddress = customer.getAddress();
   Address shippingAddress = customer.getAddress();
   // Create a TShirt object
   TShirt tShirt = new TShirt(Integer.parseInt(shirtID), size, color,
    designFront, designBack, pictureFront, pictureBack, priceForItem);
   tShirt.setItemID(Integer.parseInt(itemID));
   tShirt.setQuantity(Integer.parseInt(quantity));
   tShirt.setTotalPrice(Double.parseDouble(totalPrice));
   TShirt[] items = {tShirt};
   try{
    // Create an Order and add to session
    Order order = new Order(
      Integer.parseInt(orderID),
      DateFormat.getInstance().parse(orderDate),shipInstruction,
      DateFormat.getInstance().parse(shipDate) ,
      Double.parseDouble(shipWeight),
      DateFormat.getInstance().parse(paidDate),
      billingAddress, shippingAddress, items);
      session.setAttribute(Order.ORDER_OBJECT, order);
    }catch(java.text.ParseException parseException){
      log(parseException.toString());
      parseException.printStackTrace();
    }
  }
  
/**
  * Generate HTML body with Orders for this customer
  */
   private void showUserOrders(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException , IOException
   {
     log(" Showing The Orders:" );

    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write top half of page
    ServletsHelper.writePageTopArea(printWriter);
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"300%\" >");
    printWriter.println(" <TR> ");
    // Create an order associated with this customer
    this.constructOrderForThisCustomer(servletRequest);
    // Write left hand side of page (buttons)
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Get Order from session and print as HTML table in center of page
    String message = "Send Password  in Email to :";
    HttpSession session = servletRequest.getSession();
    Order order = (Order)session.getAttribute(Order.ORDER_OBJECT);
    printWriter.println("<TD VALIGN=TOP>" );
    printWriter.println(order.getOrderAsHTMLTable());
    printWriter.println(" </TD> ");
    printWriter.println("</TR>");
    printWriter.println("</TABLE>");
    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
   }


}
