package ejava.servlets;

// 
import java.io.IOException;
import java.io.PrintWriter;

// 
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
  * This servlet is used to handle user registration requests
*/
public class RegistrationServlet extends HttpServlet
{

  public final static String REGISTRATION_HTML = "/registration.html";
  /** 6 Months in seconds **/
  public static int  SIX_MONTHS_IN_SECONDS = 60*60*24*183;

  public void service(HttpServletRequest servletRequest,
    HttpServletResponse servletResponce)
    throws ServletException, IOException
  {
    doPost(servletRequest, servletResponce);
  }

/**
  * Handles a POST request for registration
  */
  public void doPost(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    boolean registered = false;
    // Make Customer session object and get cookie
    try{
      // Create Customer object in session from POST data
      String firstName = registerNewCustomer(servletRequest);
      // Get session object and the cookie name value
      HttpSession session = servletRequest.getSession(false);
      String cookieValue
          = (String)session.getAttribute(ServletsHelper.COOKIE_NAME);
      // If no cookie, then add user's first name to cookie
      if(cookieValue == null){
        // create Cookie for this customer
        Cookie cookie = new Cookie(ServletsHelper.COOKIE_NAME, firstName);
        cookie.setMaxAge(SIX_MONTHS_IN_SECONDS);
        servletResponse.addCookie(cookie);
      }
      registered = true;
    }
    catch(ServletException servletException){
      log("Failed to Register :" + servletException);
    }

    // Generate registration success or failure information 
    sendSuccessOrFailInforamtion(registered, servletRequest, servletResponse);
  }

/**
  * Creates Customer object from post data and adds to session 
  */
  private String registerNewCustomer(HttpServletRequest servletRequest)
    throws ServletException
  {
    // Extract all Post data into Java objects
    String firstName =
      (String)servletRequest.getParameterValues(Customer.FIRST_NAME)[0];
    String lastName =
      (String)servletRequest.getParameterValues(Customer.LAST_NAME)[0];
    String[] values =
      servletRequest.getParameterValues(Customer.MIDDLE_NAME);
    String middleName  = null;
    if(values != null){
      middleName =
      (String)servletRequest.getParameterValues(Customer.MIDDLE_NAME)[0];
    }
    String password =
     (String)servletRequest.getParameterValues(Customer.PASSWORD)[0];
    String phone =
      (String)servletRequest.getParameterValues(Customer.PHONE)[0];
    String eMail =
      (String)servletRequest.getParameterValues(Customer.EMAIL)[0];
    String address_1 =
      (String)servletRequest.getParameterValues(Address.ADDRESS_1)[0];
    String address_2 =
      (String)servletRequest.getParameterValues(Address.ADDRESS_2)[0];
    String city =
      (String)servletRequest.getParameterValues(Address.CITY)[0];
    String state =
      (String)servletRequest.getParameterValues(Address.STATE)[0];
    String zip_code =
      (String)servletRequest.getParameterValues(Address.ZIP_CODE)[0];

    // Create an Address object from POST data
    Address address = new Address(address_1, address_2, city,
                                  state,zip_code);
    // Create a Customer object from Address object and POST data
    Customer customer = new Customer(firstName, lastName, middleName,
                                     address, phone, eMail, password);
    // Get session and add Customer object to session
    HttpSession session  = servletRequest.getSession();
    session.setAttribute(Customer.CUSTOMER_OBJECT, customer);

    // Return user's first name
    return firstName;
  }

/**
  * Send fail/success information to Client.
  */
  private void sendSuccessOrFailInforamtion(boolean registered, HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException, IOException
  {

    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write page body
    writePageBody(registered, printWriter, servletRequest);
    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

  /**
  * Write Page Body
  */
  private void writePageBody(boolean registered, PrintWriter printWriter,
    HttpServletRequest servletRequest)
  {
    // Write top half of page
    ServletsHelper.writePageTopArea(printWriter);
    // Divide rest of page into two table columns
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"300%\" >");
    printWriter.println(" <TR> ");
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    // Write left half of page (buttons)
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write center area of page
    writePageCenterArea(registered, printWriter, servletRequest);
    // close the TABLE
    printWriter.println("</TR>");
    printWriter.println("</TABLE> <br>&nbsp;");

  }
  
/**
  * Write the page center area with registration result
  */
  private void writePageCenterArea(boolean registered,
    PrintWriter printWriter, HttpServletRequest servletRequest)
  {
   if(registered){
    printWriter.println(" <TD  ALIGN=TOP > ");
    printWriter.println("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 "+
      " WIDTH=\"400\"  BGCOLOR=\"#FFCC66\" > ");
    printWriter.println("<TR> <TD COLSPAN=\"2\" BGCOLOR=\"#00FF00\"> "+
      " <FONT face=\"arial, sans-serif\" > "+
      " <FONT color=\"#FF0000\"> "  +
      " <FONT SIZE=-1>Register following Information : "  +
      " </FONT> </FONT> </FONT> </B> </TD> </TR> " );
   }else {
    printWriter.println("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 "+
      " WIDTH=\"400\"  BGCOLOR=\"#FFCC66\" > ");
    printWriter.println("<TR> <TD COLSPAN=\"2\" BGCOLOR=\"#FF0000\"> "+
      " <FONT face=\"arial, sans-serif \"> "+
      " <FONT color=\"#FF0000\"> "  +
      " <FONT SIZE=-1>Failed to Register following Information : "  +
      " </FONT> </FONT> </FONT> </B> </TD> </TR> " );
   }
    HttpSession session = servletRequest.getSession(false);
    Customer customer =
      (Customer)session.getAttribute(Customer.CUSTOMER_OBJECT);

    printWriter.println("<TR> <TD>  "+
      Customer.FIRST_NAME + " : </TD> <TD> " + customer.getFirstName()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Customer.LAST_NAME + " : </TD> <TD> " + customer.getLastName()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Customer.MIDDLE_NAME + " : </TD> <TD> " + customer.getMiddleName()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Customer.EMAIL + " : </TD> <TD> " + customer.getEmail()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Customer.PHONE + " : </TD> <TD> " + customer.getPhone()+" </TD> </TR> " );
    Address address = customer.getAddress();
    printWriter.println("<TR> <TD>  "+
      Address.ADDRESS_1+ " : </TD> <TD> " + address.getAddress_1()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Address.ADDRESS_2+ " : </TD> <TD> " + address.getAddress_2()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Address.CITY+ " : </TD> <TD> " + address.getCity()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Address.STATE+ " : </TD> <TD> " + address.getState()+" </TD> </TR> " );
    printWriter.println("<TR> <TD>  "+
      Address.ZIP_CODE+ " : </TD> <TD> " + address.getZipCode()+" </TD> </TR> " );
    printWriter.println("</TABLE> </TD> ");

  }



}
