package ejava.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.util.Enumeration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
  * This class defines all frequently used tags in HTML as constants
  * and a few helper methods.
  */
public class ServletsHelper
{
  /** Content Type **/
  public static String CONTENT_TYPE = "text/html";
  /** Standard BeeShirts.com Cookie Name **/
  public static final String  COOKIE_NAME = "BEESHIRTS";

  public static PrintWriter createInitialDocumentResponse(
        HttpServletResponse servletResponse)
    throws IOException
  {
    // Set common text/html content type for all of our documents
    servletResponse.setContentType(CONTENT_TYPE);
    // Retrieve the PrintWriter handle
    PrintWriter printWriter = servletResponse.getWriter();
    // Print common DOCTYPE, HTML begin tag, HEAD, and TITLE tags
    printWriter.println(
      "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
      + "\n <HTML> \n <HEAD> \n <TITLE> "
      + "\n BeeShirts.com </TITLE>\n </HEAD>\n");
    // Print common BODY begin tags
    printWriter.println( "<BODY ALINK=\"#FFCC66\"  BGCOLOR=\"#ffcc66\" "+
      " LINK=\"#FFFFFF\" VLINK=\"#FFFFFF\"> " );

    return printWriter;
  }

  public static void createClosingDocumentResponse(
    PrintWriter printWriter)
  {
    printWriter.println(" </BODY> ");
    printWriter.println( "</HTML>" );
    printWriter.close();
  }


/**
  * Print ServletContext Information
  */
  public static void printServletContextInformation(
    ServletContext servletContext) throws ServletException
  {
    // Get attribute names from servlet context
    Enumeration attributes = servletContext.getAttributeNames();
    // For each attribute, print name and value to log
    while(attributes.hasMoreElements()){
      String attributeName = (String)attributes.nextElement();
      Object attribute = servletContext.getAttribute(attributeName);
      servletContext.log("Attribute :" +attributeName +
        " Attribute :" + attribute);
    }

    // Get parameter names from context
    Enumeration parameters = servletContext.getInitParameterNames();
    // For each parameter, print name and value to log
    while(parameters.hasMoreElements()){
      String parameterName = (String)parameters.nextElement();
      Object parameterValue = servletContext.getAttribute(parameterName);
      servletContext.log("Parameter :" +parameterName +
        " Parameter Value :" + parameterValue);
    }

    // Get and log real path
    String servletPath = servletContext.getRealPath("/");
    servletContext.log("Servlet Path :"+servletPath);
    // Get and log server info
    String serverInformation = servletContext.getServerInfo();
    servletContext.log("Server Information  :"+serverInformation);
    // Log major and minor version of API
    servletContext.log("Major Version :" +servletContext.getMajorVersion());
    servletContext.log("Minor Version :" +servletContext.getMinorVersion());

    // Get and log resource info
    try{
      java.net.URL resource = servletContext.getResource("/");
      servletContext.log("Resource : " +resource);
    }
    catch(MalformedURLException malformedURLException){
      servletContext.log(" Error :" +malformedURLException);
    }
  }

/**
  * Prints information about the request.
  */
/*
  public static void printRequestInformation(
    HttpServletRequest servletRequest, ServletContext servletContext)
  {
    int requestContentLength = servletRequest.getContentLength();
    servletContext.log("Request Length :" +requestContentLength);

    String contentType = servletRequest.getContentType();
    servletContext.log("Request Content Type :" + contentType);

    String protocol = servletRequest.getProtocol();
    servletContext.log("Servlet Protocol :" + protocol);

    String schemeName = servletRequest.getScheme();
    servletContext.log("Scheme Name :" +schemeName);

    String serverName = servletRequest.getServerName();
    servletContext.log("Server Name :" + serverName);

    int serverPort = servletRequest.getServerPort();
    servletContext.log("Server Port :"+serverPort);

    String remoteAddress = servletRequest.getRemoteAddr();
    servletContext.log("Remote Addres :"+remoteAddress);

    String characterEncoding = servletRequest.getCharacterEncoding();
    servletContext.log(" Request Character Encoding :" +characterEncoding);

    String attributeValue = (String)servletRequest.getAttribute("CONFIG_FILE");
    servletContext.log(" Attribute : CONFIG_FILE  Value :" + attributeValue);

    String url = servletRequest.getRequestURL().toString();
    servletContext.log(" URL :" + url);

    String remoteUser = (String)servletRequest.getRemoteUser();
    servletContext.log("Remote User :" +remoteUser);

    String queryString = servletRequest.getQueryString();
    servletContext.log("Query String :" + queryString);

    String requestedURI = servletRequest.getRequestURI();
    servletContext.log("Request URI : " + requestedURI);

    String remoteHost = servletRequest.getRemoteHost();
    servletContext.log("Remote Host : " + remoteHost);

    // Get request parameter names and log each name and value 
    Enumeration enumeration = servletRequest.getParameterNames();
    while(enumeration.hasMoreElements()){
      String parameter = (String)enumeration.nextElement();
      String[] parameterValues = servletRequest.getParameterValues(parameter);
      servletContext.log("Parameter :" +parameter +" values :" +parameterValues);
    }

    // Get request header names and log each name and value 
    Enumeration headerNames = servletRequest.getHeaderNames();
    while (headerNames.hasMoreElements()) {
      String name = (String)headerNames.nextElement();
      String value = servletRequest.getHeader(name);
      servletContext.log("Name :" + name +" Value :" +value);
    }
  }
*/

/**
  * Print servletConfig information
  */
  public static void printServletConfigInformation(
    ServletConfig servletConfig) throws ServletException
  {
    // Retrieve servlet context and log message
    ServletContext servletContext = servletConfig.getServletContext();
    servletContext.log("Servlet Initial Parameters Names:");
    // Retrieve parameter names from ServletConfig
    Enumeration enumeration = servletConfig.getInitParameterNames();
    // For each parameter, log the name and value
    while(enumeration.hasMoreElements()){
      String parameterName = (String)enumeration.nextElement();
      String value = servletConfig.getInitParameter(parameterName);
      servletContext.log("Parameter :" +parameterName +
        " Parameter Value :"+value);
    }

    // Now print the ServletContext information
    servletContext.log("Servlet Context information :");
    printServletContextInformation(servletConfig.getServletContext());
  }

/**
  * For all of our pages the Top of the screen is same
  */
  public static void writePageTopArea(PrintWriter printWriter)
  {
   // first set the logo images on the top of the page
    printWriter.println("<IMG SRC=\"logotitle.gif\" "+
      "  HEIGHT=70 WIDTH=312> "+
      " <IMG SRC=\"logoshirt.jpg\" HEIGHT=71 WIDTH=232> ");
    printWriter.println("<BR> ");
    printWriter.println("<BR> ");
    printWriter.println("<HR> ");
  }

/**
  * Get a String of server information frmo servlet request
  */
  public static String getServerInfo(HttpServletRequest servletRequest)
  {
    String serverHost = servletRequest.getServerName();
    int serverPort = servletRequest.getServerPort();
    String context = servletRequest.getContextPath();

    return serverHost +":"+serverPort + context;
  }

/**
  * For all of our pages the left side of the screen is the same
  */
  public static void writePageLeftArea(PrintWriter printWriter,
    String serverInfo)
  {
    // first divide the remaining area into a table and
    // fill the left area as a row of buttons
    printWriter.println("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 "+
      "WIDTH=\"640\" >" );
    printWriter.println("<TD VALIGN=\"TOP\"> ");
    // left side area is a single cell with another table
    printWriter.println("<TABLE BORDER=0 COLS=1 WIDTH=\"110\"  "+
      "BGCOLOR= \"#ffcc66\"> ");
    printWriter.println("<TR>");
    // Make links to buttons
    // about us information
    printWriter.println("<TD> "+
      " <A HREF=\"http://"+serverInfo+AboutUsServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"aboutus.gif\" HEIGHT=22 WIDTH=110  "+
      " ALT=\"Click Here to Browse Shirts\"> </A> " );
    printWriter.println("</TD> </TR>");
    // contact us information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\">"+
      "<A HREF=\"http://"+serverInfo+ContactInformationServlet.SERVLET_NAME+"\"> " +
      "<IMG SRC=\"contact.gif\" HEIGHT=22 WIDTH=110  "+
      " ALT=\"Click To Know How to Contact Us \"> </A>");
    printWriter.println(" </TD> </TR>");
    // registration information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+"/registration.html\"> " +
      " <IMG SRC=\"join.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to Register your information\"> </A>"+
      " </TD> </TR> ");
    // browse shirts information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+BrowseShirtsServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"browse.gif\" HEIGHT=22 WIDTH=110 "+
      " ALT=\"Click to Browse shirts\"> </A>"+
      " </TD> </TR> ");
    // cart information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+CartServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"viewCart.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to see Frequenty Asked Questions\"> </A>"+
      " </TD> </TR> ");
    // check out Informaion
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+CheckOutServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"checkout.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to know about Check Out Information\"> </A>"+
      " </TD> </TR> ");
    // order status information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+OrderStatusServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"orderstatus.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to get Order Status Information\"> </A>"+
      " </TD> </TR> ");
    // terms and conditions
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+TermsAndConditionsServlet.SERVLET_NAME+"\"> " +
      " <IMG SRC=\"terms.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to know about terms and conditions\"> </A>"+
      " </TD> </TR> ");
    // delivery information
    printWriter.println("<TR>");
    printWriter.println("<TD valign=\"left\"> "+
      " <A HREF=\"http://"+serverInfo+"/deliveryinformation.html\"> " +
      " <IMG SRC=\"delivery.gif\" HEIGHT=22 WIDTH=110 "+
      "ALT=\"Click to know about Delivery Information\"> </A>"+
      " </TD> </TR> ");
      
    // close this table
    printWriter.println("</TABLE>");
    // close this Table data
    printWriter.println("</TD>");

  }

/**
  * Check whether the standard BEESHIRTS Cookie is available
  */
  public static String getOurCookieValue(Cookie[] cookies)
  {
    String returnValue = null;
    // Search cookie list for standard BEESHIRTS cookie
    for(int i = 0; i<cookies.length; i++){
      Cookie cookie = cookies[i];
      // If retrieved cookie name is standard cookie name: "BEESHIRTS"
      if(cookie.getName().equals(COOKIE_NAME)){
        // Then set return value equal to value of this cookie
        returnValue = cookie.getValue();
        // Now return cookie value from this method
        return  returnValue;
      }
    }
    // Will return null cookie value if made it to this point
    return returnValue;
  }

/**
  * Print All Cookies information.
  */
  public static void printCookiesInformation(Cookie[] cookies,
    ServletContext servletContext)
  {

   for(int i = 0; i<cookies.length; i++){
    Cookie cookie = cookies[i];
    String cookieComment = cookie.getComment();
    servletContext.log("Cookie Comment :" +cookieComment);
    String cookieDomain = cookie.getDomain();
    servletContext.log("Cookie Domain :" +cookieDomain);
    int cookieAge = cookie.getMaxAge();
    servletContext.log("Cookie Age :" +cookieDomain);
    String cookiePath = cookie.getPath();
    servletContext.log("Cookie Path :" +cookiePath);
    boolean cookieSecurity = cookie.getSecure();
    servletContext.log("Cookie Security :" +cookieSecurity);
    String cookieName = cookie.getName();
    servletContext.log("Cookie Name :" +cookieName);
    String cookieValue = cookie.getValue();
    servletContext.log("Cookie Value :" +cookieValue);
    int cookieVersion = cookie.getVersion();
    servletContext.log("Cookie Version :" +cookieVersion);
   }
  }
}
