package ejava.tcpip;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.ServerSocket;
import java.util.Vector;
import java.util.Hashtable;

/**
 * ClientReceiverServer, it is used to receive the
 * server response for the orders.
 * 
 */
public class ClientReceiverServer extends Thread 
    implements QueryServiceProtocol {
    private Socket             socket;
    private ObjectInputStream  remoteInStream;
    private ObjectOutputStream remoteOutStream;
    private int                portNumber = 5556;
    ServerSocket               serverSocket = null;

    public ClientReceiverServer(int portNumber) throws IOException {
        if (portNumber > -1) {
            this.portNumber = portNumber;
        } 

        serverSocket = new ServerSocket(this.portNumber, 1);
    }

    public synchronized void run() {
        boolean listening = true;

        /**
         * Until finalize method of this class is called by GC
         * it waits for the server to write .
         */
        while (listening) {
            try {

                // Block on socket and read data when connection made
                Socket socket = serverSocket.accept();

                remoteInStream = 
                    new ObjectInputStream(socket.getInputStream());

                Vector receivedOrders = 
                    (Vector) this.remoteInStream.readObject();

                // For each vector received, create an Order object
                for (int i = 0; i < receivedOrders.size(); i++) {
                    Order order = (Order) receivedOrders.elementAt(i);

                    System.out.println("Received the Orders :");
                    System.out.println("Order Data :" 
                                       + order.getOrderDate());
                    System.out.println("Order ID :" 
                                       + order.getOrderID());
                    System.out.println("Value :" 
                                       + order.getOrderValue());
                } 
            } catch (IOException ioException) {
                System.out.println("Error :" + ioException);
                ioException.printStackTrace();
            } catch (Exception exception) {
                System.out.println("Exception :" + exception);
                exception.printStackTrace();
            } 
        } 
    } 

    protected void finalize() throws Throwable {
        try {
            if (remoteInStream != null) {
                this.remoteInStream.close();
            } 

            if (this.remoteOutStream != null) {
                this.remoteOutStream.close();
            } 
        } catch (IOException ex) {}

        super.finalize();
    } 

}

