package ejava.tcpip;

import java.net.Socket;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.InetAddress;

/**
 * Once server opens socket and given connections
 * to the ServerHelper, the serverhelper
 * start listening for client requests
 */
public class ServerHelper extends Thread
    implements QueryServiceProtocol {
    private Socket             socket;
    private ObjectInputStream  remoteInStream;
    private ObjectOutputStream remoteOutStream;
    private String             clientReceiverServerHost;
    private int                clientReceiverPort;
    private DatabaseHelper     databaseHelper;

    public ServerHelper(Socket socket,
                        DatabaseHelper dbHelper) throws IOException {
        this.socket = socket;

        this.displayClientInformation();

        this.databaseHelper = dbHelper;
        this.remoteOutStream =
            new ObjectOutputStream(socket.getOutputStream());
        this.remoteInStream =
            new ObjectInputStream(socket.getInputStream());
    }

    private void displayClientInformation() {
        System.out.println(" I am serving Client :");

        InetAddress address = socket.getInetAddress();

        System.out.println(" at Host " + address.getHostName());
        System.out.println(" IP address :"
                           + address.getHostAddress());
    }

    public synchronized void run() {
        boolean listening = true;

        /**
         * Until finalize method of this class is invoked by Garbage
         * Collector, it waits for client requests
         */
        while (listening) {
            try {
                int requested = remoteInStream.read();

                switch (requested) {

                case CUSTOMERS_QUERY: {
                    float  queryValue =
                        this.remoteInStream.readFloat();
                    Vector customers = this.getCustomers(queryValue);

                    this.remoteOutStream.writeObject(customers);

                    break;
                }

                case ORDERS_QUERY: {
                    String state =
                        (String) this.remoteInStream.readObject();

                    this.findOrdersWhichAreFrom(state);

                    break;
                }

                case REGISTER_RECEIVER: {
                    String hostName =
                        (String) this.remoteInStream.readObject();
                    int    portNumber = this.remoteInStream.readInt();

                    this.registerReceiver(hostName, portNumber);

                    break;
                }
                }
            } catch (IOException ioException) {
                System.out.println("Error :" + ioException);
                ioException.printStackTrace();

                try {
                    System.in.read();
                    clean();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            } catch (Exception exception) {
                System.out.println("Exception :" + exception);
                exception.printStackTrace();

                try {
                    System.in.read();
                    clean();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    private Vector getCustomers(float chargeValue) throws Exception {
        return this.databaseHelper
            .findCustomersWhoOrderdForMoreThan(chargeValue);
    }

    private void findOrdersWhichAreFrom(String state)
            throws Exception {

        // Get Vector of orders, get Inet Address, create callback socket,
        // and write orders to client callback
        Vector orders =
            this.databaseHelper.findOrdersWhichAreFrom(state);

        if (this.clientReceiverServerHost != null
                && this.clientReceiverServerHost.indexOf("local")
                   != -1) {
            this.clientReceiverServerHost = null;
        }

        InetAddress        serverAddress =
            InetAddress.getByName(this.clientReceiverServerHost);
        Socket             socket =
            new Socket(serverAddress, this.clientReceiverPort);
        ObjectOutputStream clientOutStream =
            new ObjectOutputStream(socket.getOutputStream());

        clientOutStream.writeObject(orders);
        clientOutStream.flush();
    }

    private void registerReceiver(String hostName, int port) {
        this.clientReceiverServerHost = hostName;
        this.clientReceiverPort = port;
    }

    /**
     * client resources
     */
    public void clean() throws IOException {
        remoteInStream.close();
        remoteOutStream.close();
        socket.close();
    }

    protected void finalize() throws Throwable {
        try {
            if (remoteOutStream != null) {
                this.remoteOutStream.close();
            }

            if (this.remoteInStream != null) {
                this.remoteInStream.close();
            }
        } catch (IOException ex) {}

        super.finalize();
    }

}

