package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: Customer
 * 
 */

import java.util.*;
import java.sql.*;

public class Customer extends SOAPHandler{

    private static final String CUSTOMER_QUERY =
        "SELECT C.* FROM TSHIRTS.CUSTOMER C, TSHIRTS.ORDERS O, "
        + "TSHIRTS.ITEM I WHERE C.CUSTOMER_ID = O.CUSTOMER_ID_FK AND "
        + "O.ORDER_ID = I.ORDER_ID_FK AND I.TOTAL_PRICE > ?";

    
    public HashMap process(HashMap inParams) throws Exception{
        
        HashMap result = new HashMap();
        String method = (String) inParams.get("beeshirts:METHOD_NAME");
        
        if(method.equals("findCustomers")){
            float f = Float.parseFloat(
                (String) inParams.get("minimumOrder"));
            Vector customers = this.findCustomers(f);
            result.put("customers", customers);
        }
        else{
            throw new Exception("Handler not found");
        }
        
        return result;
    }
    
    public Vector findCustomers(float minimumOrder){

        Vector customers = null;        
        
        try{
            // Get result set, meta data, and column count
            DatabaseHelper dbh = new DatabaseHelper();
            ResultSet rs = dbh.getResultSet(CUSTOMER_QUERY,
                                        new Float(minimumOrder));
            ResultSetMetaData rsMetaData = rs.getMetaData();
            
            // Create return Vector
            customers = new Vector(rsMetaData.getColumnCount());
    
            // For each result, populate customer object with data
            while (rs.next()) {
                
                HashMap customer = new HashMap();
                    
                customer.put("customerID", rs.getString("CUSTOMER_ID"));
                customer.put("firstName", rs.getString("FIRST_NAME"));
                customer.put("lastName", rs.getString("LAST_NAME"));
                customer.put("address1", rs.getString("ADDRESS_1"));
                customer.put("address2", rs.getString("ADDRESS_2"));
                customer.put("city", rs.getString("CITY"));
                customer.put("state", rs.getString("STATE"));
                customer.put("phoneNumber", rs.getString("PHONE"));
                customer.put("eMail", rs.getString("EMAIL"));
                customer.put("zip", rs.getString("ZIP_CODE"));

                customer.put("hasAttachments", "false");                
                customers.add(customer);
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }

        return customers;
        
    }
    
}
