package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: Order
 * 
 */

import java.util.*;
import java.io.*;
import java.sql.*;

public class Order extends SOAPHandler{

    private static final String ORDER_QUERY =
        "SELECT ORDERS.order_id, ORDERS.order_date, ITEM.total_price"
        + " FROM TSHIRTS.ORDERS, TSHIRTS.ITEM, TSHIRTS.CUSTOMER WHERE "
        + "ORDERS.order_id = ITEM.order_id_fk AND "
        + "ORDERS.CUSTOMER_ID_FK = CUSTOMER.CUSTOMER_ID "
        + "AND CUSTOMER.state = ?";

    public HashMap process(HashMap inParams) throws Exception{
        
        HashMap result = new HashMap();
        String method = (String) inParams.get("beeshirts:METHOD_NAME");
        
        if(method.equals("findOrders")){
            String state = (String) inParams.get("state");
            Vector orders = this.findOrders(state);
            result.put("orders", orders);
        }
        else if(method.equals("getCoupon")){
            String couponID = (String) inParams.get("couponID");
            Vector coupons = this.getCoupon(couponID);
            result.put("coupons", coupons);
        }        
        else{
            throw new Exception("Handler not found");
        }
        
        return result;
    }


    public synchronized Vector findOrders(String state){
        Vector orders = null;
        
        try{
    
            // Get result set from query, and column count
            DatabaseHelper dbh = new DatabaseHelper();
            ResultSet rs = dbh.getResultSet(ORDER_QUERY, state);
            ResultSetMetaData rsMetaData = rs.getMetaData();
            orders = new Vector(rsMetaData.getColumnCount());
    
            // For each result, populate and Order object
            while (rs.next()) { 
                
                HashMap order = new HashMap();
                
                order.put("orderID", 
                    String.valueOf(rs.getInt("ORDER_ID")));
                order.put("orderDate",
                    rs.getDate("ORDER_DATE").toString());
                order.put("orderValue",
                    String.valueOf(rs.getDouble("TOTAL_PRICE")));
                    
                order.put("hasAttachments", "false");
                orders.addElement(order);
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }

        return orders;
    }
    
    public synchronized Vector getCoupon(String couponID){
        Vector coupons = null;
        
        try{
            coupons = new Vector(1);
                
            HashMap couponData = new HashMap();
            HashMap couponAttachments = new HashMap();
            HashMap coupon = new HashMap(3);
            
            couponData.put("couponID", couponID);                                  

            couponAttachments.put("couponImage", "coupon.jpg"); 
                
            coupon.put("hasAttachments", "true");
            coupon.put("data", couponData);
            coupon.put("attachments", couponAttachments);
            
            coupons.addElement(coupon);
        }
        catch(Exception e){
            e.printStackTrace();
        }

        return coupons;
    }    

}
