package ejava.webservices.rpc;

/**
 * @author Paul Perrone
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices.rpc
 * 
 * Type: AdminHandler
 * 
 * Implements a security handler on the server side.
 * 
 */

import javax.xml.rpc.handler.*;
import javax.xml.rpc.handler.soap.*;
import javax.xml.soap.*;

public class AdminHandler extends SecurityHandler {

  protected String username = "";
  protected String password = "";

  public AdminHandler(){
  }

  public void init(HandlerInfo config){
  	// Read parent security header and the admin username
  	//   and password from the runtime configuration
    super.init(config);
    java.util.Map configMap = config.getHandlerConfig();
    username = (String) configMap.get(USERNAME);
    password = (String) configMap.get(PASSWORD);
  }

  public boolean handleRequest(MessageContext ctx){
  	// Handle the request by checking the username/password
    boolean status = false;
    String givenUsername = "";
    String givenPassword = "";
    try{
      System.out.println("  -- HANDLE REQUEST ON SERVER --");

      // Get SOAP Message Context and Envelope
      SOAPMessageContext soapCtx = (SOAPMessageContext) ctx;
      SOAPMessage message = soapCtx.getMessage();
      SOAPPart part = message.getSOAPPart();
      SOAPEnvelope envelope = part.getEnvelope();
      
      // Get SOAP header
      SOAPHeader header = envelope.getHeader();

      // Print what is received as part of the header
      //    (look in Apache's catalina.log file)
      super.printSOAPInfo(header);
      
      // Get SOAP Header security info
      SOAPHeaderElement theHeader
        = (SOAPHeaderElement) header.getChildElements().next();
      Name securityHeaderName
        = SOAPFactory.newInstance().createName(NAME, PREFIX, NAMESPACE);
      Name usernameName
        = SOAPFactory.newInstance().createName(USERNAME);
      givenUsername
        = (String)((SOAPElement)
        theHeader.getChildElements(usernameName).next()).getValue();
      Name passwordName
        = SOAPFactory.newInstance().createName(PASSWORD);
      givenPassword
        = (String)((SOAPElement)
          theHeader.getChildElements(passwordName).next()).getValue();

	 // Compare the username/password given by the client
	 //   to the admin username/password read by the init() method
     if(givenUsername.equals(username) &&
          givenPassword.equals(password)){
        status = true;
      }
      else{
        status = false;
      }
    }
    catch(Exception e){
     e.printStackTrace();
    }
     return status;
  }
}
